@@ -31,6 +31,7 @@ while(<PWFILE>) {
 		$user{'notabs'} = $gconfig{"notabs_$user[0]"};
 		$user{'skill'} = $gconfig{"skill_$user[0]"};
 		$user{'risk'} = $gconfig{"risk_$user[0]"};
+		$user{'rbacdeny'} = $gconfig{"rbacdeny_$user[0]"};
 		$user{'theme'} = $gconfig{"theme_$user[0]"};
 		$user{'ownmods'} = [ split(/\s+/,
 					   $gconfig{"ownmods_$user[0]"}) ];
@@ -96,7 +97,7 @@ elsif (defined($user{'theme'})) {
 &unlock_file($ENV{'MINISERV_CONFIG'});
 
 &lock_file($miniserv{'userfile'});
-open(PWFILE, ">> $miniserv{'userfile'}");
+open(PWFILE, ">>$miniserv{'userfile'}");
 print PWFILE "$user{'name'}:$user{'pass'}:$user{'sync'}:$user{'cert'}:",
 	$user{'allow'} ? "allow $user{'allow'}" :
 	$user{'deny'} ? "deny $user{'deny'}" : "","\n";
@@ -105,7 +106,7 @@ close(PWFILE);
 
 &lock_file(&acl_filename());
 @mods = &list_modules();
-open(ACL, ">> ".&acl_filename());
+open(ACL, ">>".&acl_filename());
 print ACL &acl_line(\%user, \@mods);
 close(ACL);
 &unlock_file(&acl_filename());
@@ -118,6 +119,8 @@ delete($gconfig{"skill_".$user{'name'}});
 $gconfig{"skill_".$user{'name'}} = $user{'skill'} if ($user{'skill'});
 delete($gconfig{"risk_".$user{'name'}});
 $gconfig{"risk_".$user{'name'}} = $user{'risk'} if ($user{'risk'});
+delete($gconfig{"rbacdeny_".$user{'name'}});
+$gconfig{"rbacdeny_".$user{'name'}} = $user{'rbacdeny'} if ($user{'rbacdeny'});
 delete($gconfig{"ownmods_".$user{'name'}});
 $gconfig{"ownmods_".$user{'name'}} = join(" ", @{$user{'ownmods'}})
 	if (@{$user{'ownmods'}});
@@ -162,17 +165,20 @@ else {
 open(PWFILE, $miniserv{'userfile'});
 @pwfile = <PWFILE>;
 close(PWFILE);
-open(PWFILE, "> $miniserv{'userfile'}");
+&open_tempfile(PWFILE, ">$miniserv{'userfile'}");
 foreach (@pwfile) {
 	if (/^([^:]+):/ && $1 eq $_[0]) {
-		print PWFILE "$user{'name'}:$user{'pass'}:",
-			     "$user{'sync'}:$user{'cert'}:",
+		&print_tempfile(PWFILE,
+			"$user{'name'}:$user{'pass'}:",
+			"$user{'sync'}:$user{'cert'}:",
 			$user{'allow'} ? "allow $user{'allow'}" :
-			$user{'deny'} ? "deny $user{'deny'}" : "","\n";
+			$user{'deny'} ? "deny $user{'deny'}" : "","\n");
+		}
+	else {
+		&print_tempfile(PWFILE, $_);
 		}
-	else { print PWFILE $_; }
 	}
-close(PWFILE);
+&close_tempfile(PWFILE);
 &unlock_file($miniserv{'userfile'});
 
 &lock_file(&acl_filename());
@@ -180,14 +186,16 @@ close(PWFILE);
 open(ACL, &acl_filename());
 @acl = <ACL>;
 close(ACL);
-open(ACL, "> ".&acl_filename());
+&open_tempfile(ACL, ">".&acl_filename());
 foreach (@acl) {
 	if (/^(\S+):/ && $1 eq $_[0]) {
-		print ACL &acl_line($_[1], \@mods);
+		&print_tempfile(ACL, &acl_line($_[1], \@mods));
+		}
+	else {
+		&print_tempfile(ACL, $_);
 		}
-	else { print ACL $_; }
 	}
-close(ACL);
+&close_tempfile(ACL);
 &unlock_file(&acl_filename());
 
 delete($gconfig{"lang_".$_[0]});
@@ -198,6 +206,8 @@ delete($gconfig{"skill_".$_[0]});
 $gconfig{"skill_".$user{'name'}} = $user{'skill'} if ($user{'skill'});
 delete($gconfig{"risk_".$_[0]});
 $gconfig{"risk_".$user{'name'}} = $user{'risk'} if ($user{'risk'});
+delete($gconfig{"rbacdeny_".$_[0]});
+$gconfig{"rbacdeny_".$user{'name'}} = $user{'rbacdeny'} if ($user{'rbacdeny'});
 delete($gconfig{"ownmods_".$_[0]});
 $gconfig{"ownmods_".$user{'name'}} = join(" ", @{$user{'ownmods'}})
 	if (@{$user{'ownmods'}});
@@ -241,22 +251,26 @@ delete($miniserv{"preroot_".$_[0]});
 open(PWFILE, $miniserv{'userfile'});
 @pwfile = <PWFILE>;
 close(PWFILE);
-open(PWFILE, "> $miniserv{'userfile'}");
+&open_tempfile(PWFILE, ">$miniserv{'userfile'}");
 foreach (@pwfile) {
-	if (!/^([^:]+):/ || $1 ne $_[0]) { print PWFILE $_; }
+	if (!/^([^:]+):/ || $1 ne $_[0]) {
+		&print_tempfile(PWFILE, $_);
+		}
 	}
-close(PWFILE);
+&close_tempfile(PWFILE);
 &unlock_file($miniserv{'userfile'});
 
 &lock_file(&acl_filename());
 open(ACL, &acl_filename());
 @acl = <ACL>;
 close(ACL);
-open(ACL, "> ".&acl_filename());
+&open_tempfile(ACL, ">".&acl_filename());
 foreach (@acl) {
-	if (!/^([^:]+):/ || $1 ne $_[0]) { print ACL $_; }
+	if (!/^([^:]+):/ || $1 ne $_[0]) {
+		&print_tempfile(ACL, $_);
+		}
 	}
-close(ACL);
+&close_tempfile(ACL);
 &unlock_file(&acl_filename());
 
 delete($gconfig{"lang_".$_[0]});
@@ -294,8 +308,8 @@ close(GROUP);
 
 if ($_[1]) {
 	foreach $m ("", &list_modules()) {
-		local $file = "$config_directory/$m/$_[1].acl";
-		local $dest = "$config_directory/$m/$_[0]->{'name'}.acl";
+		local $file = "$config_directory/$m/$_[1].gacl";
+		local $dest = "$config_directory/$m/$_[0]->{'name'}.gacl";
 		if (-r $file) {
 			local %macl;
 			&read_file($file, \%macl);
@@ -320,9 +334,9 @@ foreach $l (@$lref) {
 &unlock_file("$config_directory/webmin.groups");
 
 foreach $m (@{$_[1]->{'modules'}}, "") {
-	local $file = "$config_directory/$m/$_[0].acl";
+	local $file = "$config_directory/$m/$_[0].gacl";
 	if (-r $file) {
-		rename($file, "$config_directory/$m/$_[1]->{'name'}.acl");
+		rename($file, "$config_directory/$m/$_[1]->{'name'}.gacl");
 		}
 	}
 }
@@ -336,7 +350,7 @@ local $lref = &read_file_lines("$config_directory/webmin.groups");
 @$lref = grep { !/^([^:]+):/ || $1 ne $_[0] } @$lref;
 &flush_file_lines();
 &unlock_file("$config_directory/webmin.groups");
-unlink(map { "$config_directory/$_/$_[0].acl" } &list_modules());
+unlink(map { "$config_directory/$_/$_[0].gacl" } &list_modules());
 }
 
 # group_line(&group)
@@ -401,23 +415,32 @@ if ($@) {
 # Update the members users and groups of some group
 sub update_members
 {
+local $m;
 foreach $m (@{$_[3]}) {
-	local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
-	if ($u) {
-		$u->{'modules'} = [ @{$_[2]}, @{$u->{'ownmods'}} ];
-		&modify_user($u->{'name'}, $u);
+	if ($m !~ /^\@(.*)$/) {
+		# Member is a user
+		local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
+		if ($u) {
+			$u->{'modules'} = [ @{$_[2]}, @{$u->{'ownmods'}} ];
+			&modify_user($u->{'name'}, $u);
+			}
 		}
-	local ($g) = grep { $_->{'name'} eq $m } @{$_[1]};
-	if ($g) {
-		$g->{'modules'} = [ @{$_[2]}, @{$g->{'ownmods'}} ];
-		&modify_group($g->{'name'}, $g);
-		&update_members($_[0], $_[1], $_[2], $g->{'members'});
+	else {
+		# Member is a group
+		local $gname = substr($m, 1);
+		local ($g) = grep { $_->{'name'} eq $gname } @{$_[1]};
+		if ($g) {
+			$g->{'modules'} = [ @{$_[2]}, @{$g->{'ownmods'}} ];
+			&modify_group($g->{'name'}, $g);
+			&update_members($_[0], $_[1], $g->{'modules'},
+					$g->{'members'});
+			}
 		}
 	}
 }
 
 # copy_acl_files(from, to, &modules)
-# Copy all .acl files from some user or group to another in a list of modules
+# Copy all .acl files from some user to another in a list of modules
 sub copy_acl_files
 {
 local $m;
@@ -430,21 +453,64 @@ foreach $m (@{$_[2]}) {
 	}
 }
 
+# copy_group_acl_files(from, to, &modules)
+# Copy all .acl files from some group to another in a list of modules
+sub copy_group_acl_files
+{
+local $m;
+foreach $m (@{$_[2]}) {
+	unlink("$config_directory/$m/$_[1].gacl");
+	local %acl;
+	if (&read_file("$config_directory/$m/$_[0].gacl", \%acl)) {
+		&write_file("$config_directory/$m/$_[1].gacl", \%acl);
+		}
+	}
+}
+# copy_group_user_acl_files(from, to, &modules)
+# Copy all .acl files from some group to a user in a list of modules
+sub copy_group_user_acl_files
+{
+local $m;
+foreach $m (@{$_[2]}) {
+	unlink("$config_directory/$m/$_[1].acl");
+	local %acl;
+	if (&read_file("$config_directory/$m/$_[0].gacl", \%acl)) {
+		&write_file("$config_directory/$m/$_[1].acl", \%acl);
+		}
+	}
+}
+
 # set_acl_files(&allusers, &allgroups, module, &members, &access)
+# Recursively update the ACL for all sub-users and groups of a group
 sub set_acl_files
 {
+local $m;
 foreach $m (@{$_[3]}) {
-	local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
-	local ($g) = grep { $_->{'name'} eq $m } @{$_[1]};
-	if ($u || $g) {
-		local $n = $u ? $u->{'name'} : $g->{'name'};
-		&lock_file("$config_directory/$_[2]/$n.acl");
-		&write_file("$config_directory/$_[2]/$n.acl", $_[4]);
-		chmod(0640, "$config_directory/$_[2]/$n.acl");
-		&unlock_file("$config_directory/$_[2]/$n.acl");
+	if ($m !~ /^\@(.*)$/) {
+		# Member is a user
+		local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
+		if ($u) {
+			local $aclfile =
+				"$config_directory/$_[2]/$u->{'name'}.acl";
+			&lock_file($aclfile);
+			&write_file($aclfile, $_[4]);
+			chmod(0640, $aclfile);
+			&unlock_file($aclfile);
+			}
 		}
-	if ($g) {
-		&set_acl_files($_[0], $_[1], $_[2], $g->{'members'}, $_[4]);
+	else {
+		# Member is a group
+		local $gname = substr($m, 1);
+		local ($g) = grep { $_->{'name'} eq $gname } @{$_[1]};
+		if ($g) {
+			local $aclfile =
+				"$config_directory/$_[2]/$g->{'name'}.gacl";
+			&lock_file($aclfile);
+			&write_file($aclfile, $_[4]);
+			chmod(0640, $aclfile);
+			&unlock_file($aclfile);
+			&set_acl_files($_[0], $_[1], $_[2], $g->{'members'}, $_[4]);
+			}
 		}
 	}
 }
