@@ -11,7 +11,7 @@ if ($module_info{'usermin'}) {
 	$single_user = $remote_user;
 	&switch_to_remote_user();
 	}
-$cron_temp_file = &tempname();
+$cron_temp_file = &transname();
 
 # list_cron_jobs()
 # Returns a lists of structures of all cron jobs
@@ -25,7 +25,7 @@ if (defined(@cron_jobs_cache)) {
 # read the master crontab file
 if ($config{'system_crontab'}) {
 	$lnum = 0;
-	open(TAB, $config{'system_crontab'});
+	&open_readfile(TAB, $config{'system_crontab'});
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
 			# A normal h m s d w time
@@ -64,11 +64,11 @@ if ($config{'system_crontab'}) {
 	}
 
 # read package-specific cron files
-opendir(DIR, $config{'cronfiles_dir'});
+opendir(DIR, &translate_filename($config{'cronfiles_dir'}));
 while($f = readdir(DIR)) {
 	next if ($f =~ /^\./);
 	$lnum = 0;
-	open(TAB, "$config{'cronfiles_dir'}/$f");
+	&open_readfile(TAB, "$config{'cronfiles_dir'}/$f");
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
 			push(@rv, { 'file' => "$config{'cronfiles_dir'}/$f",
@@ -106,17 +106,23 @@ if ($single_user) {
 	@users = ( $single_user );
 	}
 else {
-	opendir(DIR, $config{'cron_dir'});
+	opendir(DIR, &translate_filename($config{'cron_dir'}));
 	@users = grep { !/^\./ } readdir(DIR);
 	closedir(DIR);
 	}
 foreach $f (@users) {
 	next if (!(@uinfo = getpwnam($f)));
 	$lnum = 0;
-	open(TAB, $single_user ? "$config{'cron_user_get_command'} |" :
-		  $fcron ?
-		  &user_sub($config{'cron_get_command'}, $f)." |" :
-		  "$config{'cron_dir'}/$f");
+	if ($single_user) {
+		&open_execute_command(TAB, $config{'cron_user_get_command'}, 1);
+		}
+	elsif ($fcron) {
+		&open_execute_command(TAB,
+			&user_sub($config{'cron_get_command'}, $f), 1);
+		}
+	else {
+		&open_readfile(TAB, "$config{'cron_dir'}/$f");
+		}
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
 			# A normal m h d m wd time
@@ -197,14 +203,16 @@ sub copy_cron_temp
 local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
 unlink($cron_temp_file);
 if ($single_user) {
-	system("$config{'cron_user_get_command'} >$cron_temp_file 2>/dev/null");
+	&execute_command($config{'cron_user_get_command'},
+			 undef, $cron_temp_file, undef);
 	}
 elsif ($fcron) {
-	system(&user_sub($config{'cron_get_command'},$_[0]->{'user'}).
-	       " >$cron_temp_file 2>/dev/null");
+	&execute_command(&user_sub($config{'cron_get_command'},$_[0]->{'user'}),
+			 undef, $cron_temp_file, undef);
 	}
 else {
-	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file 2>/dev/null");
+	system("cp ".&translate_filename("$config{'cron_dir'}/$_[0]->{'user'}").
+	       " $cron_temp_file 2>/dev/null");
 	}
 }
 
@@ -304,7 +312,7 @@ else {
 sub read_crontab
 {
 local(@tab);
-open(TAB, "$config{cron_dir}/$_[0]");
+&open_readfile(TAB, "$config{cron_dir}/$_[0]");
 @tab = <TAB>;
 close(TAB);
 if (@tab >= 3 && $tab[0] =~ /DO NOT EDIT/ &&
@@ -320,16 +328,17 @@ sub copy_crontab
 {
 local($pwd);
 if (`cat $cron_temp_file` =~ /\S/) {
-	local $temp = &tempname();
+	local $temp = &transname();
 	local $rv;
 	if ($config{'cron_edit_command'}) {
 		# fake being an editor
-		local $notemp = &tempname();
-		open(NO, ">$notemp");
-		print NO "No\n";
-		print NO "N\n";
-		print NO "no\n";
-		close(NO);
+		# XXX does not work in translated command mode!
+		local $notemp = &transname();
+		&open_tempfile(NO, ">$notemp");
+		&print_tempfile(NO, "No\n");
+		&print_tempfile(NO, "N\n");
+		&print_tempfile(NO, "no\n");
+		&close_tempfile(NO);
 		$ENV{"VISUAL"} = $ENV{"EDITOR"} =
 			"$module_root_directory/cron_editor.pl";
 		$ENV{"CRON_EDITOR_COPY"} = $cron_temp_file;
@@ -348,13 +357,14 @@ if (`cat $cron_temp_file` =~ /\S/) {
 	else {
 		# use the cron copy command
 		if ($single_user) {
-			$rv = system($config{'cron_user_copy_command'}.
-				     " <$cron_temp_file >$temp 2>&1");
+			$rv = &execute_command(
+				$config{'cron_user_copy_command'},
+				$cron_temp_file, $temp, $temp);
 			}
 		else {
-			$rv = system(
-				&user_sub($config{'cron_copy_command'},$_[0]).
-				" <$cron_temp_file >$temp 2>&1");
+			$rv = &execute_command(
+				&user_sub($config{'cron_copy_command'}, $_[0]),
+				$cron_temp_file, $temp, $temp);
 			}
 		}
 	local $out = `cat $temp`;
@@ -367,12 +377,11 @@ if (`cat $cron_temp_file` =~ /\S/) {
 else {
 	# No more cron jobs left, so just delete
 	if ($single_user) {
-		system($config{'cron_user_delete_command'}.
-		       " >/dev/null 2>/dev/null");
+		&execute_command($config{'cron_user_delete_command'});
 		}
 	else {
-		system(&user_sub($config{'cron_delete_command'},$_[0]).
-		       " >/dev/null 2>/dev/null");
+		&execute_command(&user_sub(
+			$config{'cron_delete_command'}, $_[0]));
 		}
 	}
 unlink($cron_temp_file);
@@ -409,7 +418,7 @@ return $tmp;
 sub list_allowed
 {
 local(@rv, $_);
-open(ALLOW, $config{cron_allow_file});
+&open_readfile(ALLOW, $config{cron_allow_file});
 while(<ALLOW>) {
 	next if (/^\s*#/);
 	chop; push(@rv, $_) if (/\S/);
@@ -424,7 +433,7 @@ return @rv;
 sub list_denied
 {
 local(@rv, $_);
-open(DENY, $config{cron_deny_file});
+&open_readfile(DENY, $config{cron_deny_file});
 while(<DENY>) {
 	next if (/^\s*#/);
 	chop; push(@rv, $_) if (/\S/);
@@ -439,9 +448,11 @@ return @rv;
 sub save_allowed
 {
 local($_);
-open(ALLOW, ">$config{cron_allow_file}");
-foreach (@_) { print ALLOW $_,"\n"; }
-close(ALLOW);
+&open_tempfile(ALLOW, ">$config{cron_allow_file}");
+foreach (@_) {
+	&print_tempfile(ALLOW, $_,"\n");
+	}
+&close_tempfile(ALLOW);
 chmod(0444, $config{cron_allow_file});
 }
 
@@ -451,9 +462,11 @@ chmod(0444, $config{cron_allow_file});
 sub save_denied
 {
 local($_);
-open(DENY, "> $config{cron_deny_file}");
-foreach (@_) { print DENY $_,"\n"; }
-close(DENY);
+&open_tempfile(DENY, "> $config{cron_deny_file}");
+foreach (@_) {
+	&print_tempfile(DENY, $_,"\n");
+	}
+&close_tempfile(DENY);
 chmod(0444, $config{cron_deny_file});
 }
 
@@ -478,7 +491,7 @@ sub save_envs
 {
 local($i, @tab, $line);
 @tab = &read_crontab($_[0]);
-open(TAB, "> $cron_temp_file");
+open(TAB, ">$cron_temp_file");
 for($i=1; $i<@_; $i+=2) {
 	print TAB "$_[$i]=$_[$i+1]\n";
 	}
@@ -496,7 +509,7 @@ sub expand_run_parts
 local $dir = $_[0];
 $dir = "$config{'run_parts_dir'}/$dir"
 	if ($config{'run_parts_dir'} && $dir !~ /^\//);
-opendir(DIR, $dir);
+opendir(DIR, &translate_filename($dir));
 local @rv = readdir(DIR);
 closedir(DIR);
 @rv = grep { !/^\./ } @rv;
@@ -718,8 +731,8 @@ $_[0]->{'weekdays'} = join(",", @wds);
 sub create_wrapper
 {
 local $perl_path = &get_perl_path();
-open(CMD, ">$_[0]");
-print CMD <<EOF;
+&open_tempfile(CMD, ">$_[0]");
+&print_tempfile(CMD, <<EOF
 #!$perl_path
 open(CONF, "$config_directory/miniserv.conf");
 while(<CONF>) {
@@ -731,7 +744,8 @@ close(CONF);
 chdir("\$root/$_[1]");
 exec("\$root/$_[1]/$_[2]", \@ARGV) || die "Failed to run \$root/$_[1]/$_[2] : \$!";
 EOF
-close(CMD);
+	);
+&close_tempfile(CMD);
 chmod(0755, $_[0]);
 }
 
@@ -814,5 +828,41 @@ else {
 	}
 }
 
+# find_cron_process(&job, [&procs])
+sub find_cron_process
+{
+local @procs;
+if ($_[1]) {
+	@procs = @{$_[1]};
+	}
+else {
+	&foreign_require("proc", "proc-lib.pl");
+	@procs = &proc::list_processes();
+	}
+local $rpd = &is_run_parts($_[0]->{'command'});
+local @exp = $rpd ? &expand_run_parts($rpd) : ();
+local $cmd = $exp[0] || $_[0]->{'command'};
+$cmd =~ s/^\s*\[.*\]\s+\&\&\s+//;
+$cmd =~ s/^\s*\[.*\]\s+\|\|\s+//;
+while($cmd =~ s/(\d*)(<|>)((\/\S+)|&\d+)\s*$//) { }
+$cmd =~ s/^\((.*)\)\s*$/$1/;
+$cmd =~ s/\s+$//;
+if ($config{'match_mode'} == 1) {
+	$cmd =~ s/\s.*$//;
+	}
+($proc) = grep { $_->{'args'} =~ /\Q$cmd\E/ &&
+		 (!$config{'match_user'} || $_->{'user'} eq $_[0]->{'user'}) }
+		@procs;
+if (!$proc && $cmd =~ /^$config_directory\/(.*\.pl)(.*)$/) {
+	# Must be a Webmin wrapper
+	$cmd = "$root_directory/$1$2";
+	($proc) = grep { $_->{'args'} =~ /\Q$cmd\E/ &&
+			 (!$config{'match_user'} ||
+			  $_->{'user'} eq $_[0]->{'user'}) }
+			@procs;
+	}
+return $proc;
+}
+
 1;
 
