@@ -85,43 +85,40 @@ sub save_command
 local $c = $_[0];
 if ($c->{'edit'}) {
 	# Save a file editor
-	&lock_file("$module_config_directory/$c->{'id'}.edit");
-	open(FILE, ">$module_config_directory/$c->{'id'}.edit");
-	print FILE $c->{'edit'},"\n";
-	print FILE $c->{'desc'},"\n";
-	print FILE $c->{'user'},"\n";
-	print FILE $c->{'group'},"\n";
-	print FILE $c->{'perms'},"\n";
-	print FILE $c->{'before'},"\n";
-	print FILE $c->{'after'},"\n";
-	print FILE $c->{'order'},"\n";
-	print FILE $c->{'usermin'},"\n";
-	print FILE $c->{'envs'},"\n";
-	close(FILE);
-	&unlock_file("$module_config_directory/$c->{'id'}.edit");
+	&open_lock_tempfile(FILE, ">$module_config_directory/$c->{'id'}.edit");
+	&print_tempfile(FILE, $c->{'edit'},"\n");
+	&print_tempfile(FILE, $c->{'desc'},"\n");
+	&print_tempfile(FILE, $c->{'user'},"\n");
+	&print_tempfile(FILE, $c->{'group'},"\n");
+	&print_tempfile(FILE, $c->{'perms'},"\n");
+	&print_tempfile(FILE, $c->{'before'},"\n");
+	&print_tempfile(FILE, $c->{'after'},"\n");
+	&print_tempfile(FILE, $c->{'order'},"\n");
+	&print_tempfile(FILE, $c->{'usermin'},"\n");
+	&print_tempfile(FILE, $c->{'envs'},"\n");
+	&close_tempfile(FILE);
 	}
 else {
 	# Save a custom command
-	&lock_file("$module_config_directory/$c->{'id'}.cmd");
-	open(FILE, ">$module_config_directory/$c->{'id'}.cmd");
-	print FILE $c->{'cmd'},"\n";
-	print FILE $c->{'desc'},"\n";
-	print FILE $c->{'user'}," ",int($c->{'raw'})," ",int($c->{'su'})," ",
+	&open_lock_tempfile(FILE, ">$module_config_directory/$c->{'id'}.cmd");
+	&print_tempfile(FILE, $c->{'cmd'},"\n");
+	&print_tempfile(FILE, $c->{'desc'},"\n");
+	&print_tempfile(FILE,
+		   $c->{'user'}," ",int($c->{'raw'})," ",int($c->{'su'})," ",
 		   int($c->{'order'})," ",int($c->{'noshow'})," ",
 		   int($c->{'usermin'})," ",int($c->{'timeout'})," ",
-		   int($c->{'clear'}),"\n";
+		   int($c->{'clear'}),"\n");
 	foreach $a (@{$c->{'args'}}) {
-		print FILE $a->{'name'},":",$a->{'type'},":",
-			   $a->{'opts'},":",$a->{'quote'},":",$a->{'desc'},"\n";
+		&print_tempfile(FILE, $a->{'name'},":",$a->{'type'},":",
+		   $a->{'opts'},":",$a->{'quote'},":",$a->{'desc'},"\n");
 		}
-	close(FILE);
-	&unlock_file("$module_config_directory/$c->{'id'}.cmd");
+	&close_tempfile(FILE);
 	}
 &lock_file("$module_config_directory/$c->{'id'}.html");
 if ($cmd->{'html'}) {
-	open(HTML, ">$module_config_directory/$c->{'id'}.html");
-	print HTML $cmd->{'html'};
-	close(HTML);
+	&open_tempfile(HTML, ">$module_config_directory/$c->{'id'}.html");
+	&print_tempfile(HTML, $cmd->{'html'});
+	&close_tempfile(HTML);
 	}
 else {
 	unlink("$module_config_directory/$c->{'id'}.html");
