@@ -10,9 +10,9 @@ if ($config{'darwin_setup'}) {
 	@local = <LOCAL>;
 	close(LOCAL);
 	$start = "WEBMIN=-";
-	open(LOCAL, ">$config{'hostconfig'}");
-	print LOCAL grep { !/^$start/ } @local;
-	close(LOCAL);
+	&open_tempfile(LOCAL, ">$config{'hostconfig'}");
+	&print_tempfile(LOCAL, grep { !/^$start/ } @local);
+	&close_tempfile(LOCAL);
 	print "Deleted from $config{'hostconfig'}\n";
 	# get rid of the startup items
 	$paramlist = "$config{'darwin_setup'}/Webmin/$config{'plist'}";
@@ -28,9 +28,9 @@ elsif (!$config{'init_base'}) {
 	@local = <LOCAL>;
 	close(LOCAL);
 	$start = "$config_directory/start";
-	open(LOCAL, ">$config{'local_script'}");
-	print LOCAL grep { !/^$start/ } @local;
-	close(LOCAL);
+	&open_tempfile(LOCAL, ">$config{'local_script'}");
+	&print_tempfile(LOCAL, grep { !/^$start/ } @local);
+	&close_tempfile(LOCAL);
 	print "Deleted from bootup script $config{'local_script'}\n";
 	}
 else {
