@@ -39,7 +39,11 @@ elsif ($action eq 'local') {
 	return $text{'log_local'};
 	}
 elsif ($action eq 'bootup') {
-	return &text('log_bootup', join(" , ", map { "<tt>$_</tt>" } keys %$p));
+	return &text('log_bootup', join(", ", map { "<tt>$_</tt>" } keys %$p));
+	}
+elsif ($action eq 'massstart' || $action eq 'massstop' ||
+       $action eq 'massenable' || $action eq 'massdisable') {
+	return &text('log_'.$action, join(", ", map { "<tt>$_</tt>" } split(/\s+/, $object)));
 	}
 else {
 	return undef;
