@@ -64,9 +64,9 @@ sub create_mount
 {
 local(@mlist, @amd, $_, $opts);
 
-open(FSTAB, ">> $config{'fstab_file'}");
+&open_tempfile(FSTAB, ">> $config{'fstab_file'}");
 &print_mount_line(@_);
-close(FSTAB);
+&close_tempfile(FSTAB);
 }
 
 # print_mount_line(directory, device, type, options, fsck_order, mount_at_boot)
@@ -83,24 +83,27 @@ if ($_[2] eq "smbfs") {
 	else {
 		($share = $_[1]) =~ s/\\/\//g;
 		}
-	print FSTAB "$share  $_[0]  $_[2]";
+	&print_tempfile(FSTAB, "$share  $_[0]  $_[2]");
 	local $roptions = &update_nsmb($share, \%options);
 	delete($roptions->{'user'});
 	$opts = &join_options($_[2], $roptions);
 	}
 else {
 	# Adding a normal mount to the fstab file
-	print FSTAB "$_[1]  $_[0]  $_[2]";
+	&print_tempfile(FSTAB, "$_[1]  $_[0]  $_[2]");
 	$opts = $_[3] eq "-" ? "" : $_[3];
 	}
 if ($_[5] eq "no") {
 	$opts = join(',' , (split(/,/ , $opts) , "noauto"));
 	}
-if ($opts eq "") { print FSTAB "  defaults"; }
-else { print FSTAB "  $opts"; }
-print FSTAB "  0  ";
-print FSTAB $_[4] eq "-" ? "0\n" : "$_[4]\n";
-close(FSTAB);
+if ($opts eq "") {
+	&print_tempfile(FSTAB, "  defaults");
+	}
+else {
+	&print_tempfile(FSTAB, "  $opts");
+	}
+&print_tempfile(FSTAB, "  0  ");
+&print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
 }
 
 
@@ -115,16 +118,18 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# Found the line to replace
 		&print_mount_line(@_[1..$#_]);
 		}
-	else { print FSTAB $_,"\n"; }
+	else {
+		&print_tempfile(FSTAB, $_,"\n");
+		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 }
 
 
@@ -139,15 +144,15 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line !~ /\S/ || $i++ != $_[0]) {
 		# Don't delete this line
-		print FSTAB $_,"\n";
+		&print_tempfile(FSTAB, $_,"\n");
 		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 }
 
 
@@ -712,7 +717,7 @@ if ($_[0] eq "nfs") {
 		}
 
 	# Try a test mount to see if filesystem is available
-	$temp = &tempname();
+	$temp = &transname();
 	mkdir($temp, 0755);
 	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
 	if ($mout =~ /No such file or directory/) {
@@ -929,7 +934,8 @@ return &join_options();
 sub create_swap
 {
 local($out, $bl);
-$bl = $_[1] * ($_[2] eq "m" ? 1024 : 1);
+$bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
+	       $_[2] eq "m" ? 1024 : 1);
 $out = `dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1`;
 if ($?) { return "dd failed : $out"; }
 $out = `mkswap $_[0] $bl 2>&1`;
