@@ -46,9 +46,9 @@ local($len, @mlist, $fsck, $dir);
 if ($_[2] eq "autofs") {
 	# An autofs mount.. add to /etc/auto_master
 	$len = grep { $_->[2] eq "autofs" } (&list_mounts());
-	open(AUTOTAB, ">> $config{autofs_file}");
-	print AUTOTAB "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n";
-	close(AUTOTAB);
+	&open_tempfile(AUTOTAB, ">> $config{autofs_file}");
+	&print_tempfile(AUTOTAB, "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n");
+	&close_tempfile(AUTOTAB);
 	}
 else {
 	# Add to the fstab file
@@ -59,9 +59,9 @@ else {
         else {
 		$fsck = $_[4];
 		}
-	open(FSTAB, ">> $config{fstab_file}");
-	print FSTAB "$_[1] $_[0] $_[2] $_[3] 0 $fsck\n";
-	close(FSTAB);
+	&open_tempfile(FSTAB, ">> $config{fstab_file}");
+	&print_tempfile(FSTAB, "$_[1] $_[0] $_[2] $_[3] 0 $fsck\n");
+	&close_tempfile(FSTAB);
 	}
 return $len;
 }
@@ -77,28 +77,32 @@ open(FSTAB, $config{fstab_file});
 close(FSTAB);
 $i = 0;
 
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# found the line not to include
 		}
-	else { print FSTAB $_,"\n"; }
+	else {
+		&print_tempfile(FSTAB, $_,"\n");
+		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 open(AUTOTAB, $config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
-open(AUTOTAB, "> $config{autofs_file}");
+&open_tempfile(AUTOTAB, "> $config{autofs_file}");
 foreach (@autotab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
 		# found line not to include..
 		}
-	else { print AUTOTAB $_,"\n"; }
+	else {
+		&print_tempfile(AUTOTAB, $_,"\n");
+		}
 	}
-close(AUTOTAB);
+&close_tempfile(AUTOTAB);
 }
 
 
@@ -112,7 +116,7 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
@@ -123,26 +127,30 @@ foreach (@fstab) {
                 	$fsck = $_[5];
                 	}
 		# Found the line to replace
-		print FSTAB "$_[2] $_[1] $_[3] $_[4] 0 $fsck\n";
+		&print_tempfile(FSTAB, "$_[2] $_[1] $_[3] $_[4] 0 $fsck\n");
+		}
+	else {
+		&print_tempfile(FSTAB, $_,"\n");
 		}
-	else { print FSTAB $_,"\n"; }
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 open(AUTOTAB, $config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
-open(AUTOTAB, "> $config{autofs_file}");
+&open_tempfile(AUTOTAB, "> $config{autofs_file}");
 foreach (@autotab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print AUTOTAB "$_[1]  $_[2]  ",
-				($_[4] eq "-" ? "" : "-$_[4]"),"\n";
+		&print_tempfile(AUTOTAB, "$_[1]  $_[2]  ",
+				($_[4] eq "-" ? "" : "-$_[4]"),"\n");
+		}
+	else {
+		&print_tempfile(AUTOTAB, $_,"\n");
 		}
-	else { print AUTOTAB $_,"\n"; }
 	}
-close(AUTOTAB);
+&close_tempfile(AUTOTAB);
 }
 
 
@@ -694,7 +702,7 @@ if ($_[0] eq "nfs") {
                 }
 
 	# Try a test mount to see if filesystem is available
-	$temp = &tempname();
+	$temp = &transname();
 	mkdir($temp, 0755);
 	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
 	if ($mout =~ /No such file or directory/) {
