@@ -53,25 +53,29 @@ local($len, @mlist, $fcsk, $dir);
 if ($_[2] eq "autofs") {
 	# An autofs mount.. add to /etc/auto_master
 	$len = grep { $_->[2] eq "autofs" } (&list_mounts());
-	open(AUTOTAB, ">> $config{autofs_file}");
-	print AUTOTAB "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n";
-	close(AUTOTAB);
+	&open_tempfile(AUTOTAB, ">> $config{autofs_file}");
+	&print_tempfile(AUTOTAB, "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n");
+	&close_tempfile(AUTOTAB);
 	&system_logged("autofs -r >/dev/null 2>&1 </dev/null");
 	}
 else {
 	# Add to the fstab file
 	$len = grep { $_->[2] ne "autofs" } (&list_mounts());
-	open(FSTAB, ">> $config{fstab_file}");
-	print FSTAB "$_[1]  $_[0]  $_[2]";
+	&open_tempfile(FSTAB, ">> $config{fstab_file}");
+	&print_tempfile(FSTAB, "$_[1]  $_[0]  $_[2]");
 	local $opts = $_[3] eq "-" ? "" : $_[3];
 	if ($_[5] eq "no") {
 		$opts = join(',' , (split(/,/ , $opts) , "noauto"));
 		}
-	if ($opts eq "") { print FSTAB "  defaults"; }
-	else { print FSTAB "  $opts"; }
-	print FSTAB "  0  ";
-	print FSTAB $_[4] eq "-" ? "0\n" : "$_[4]\n";
-	close(FSTAB);
+	if ($opts eq "") {
+		&print_tempfile(FSTAB, "  defaults");
+		}
+	else {
+		&print_tempfile(FSTAB, "  $opts");
+		}
+	&print_tempfile(FSTAB, "  0  ");
+	&print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
+	&close_tempfile(FSTAB);
 	}
 return $len;
 }
@@ -87,21 +91,23 @@ open(FSTAB, $config{fstab_file});
 close(FSTAB);
 $i = 0;
 
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# found the line not to include
 		}
-	else { print FSTAB $_,"\n"; }
+	else {
+		&print_tempfile(FSTAB, $_,"\n");
+		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 local $found;
 open(AUTOTAB, $config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
-open(AUTOTAB, "> $config{autofs_file}");
+&open_tempfile(AUTOTAB, "> $config{autofs_file}");
 foreach (@autotab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
@@ -109,9 +115,11 @@ foreach (@autotab) {
 		$line =~ /^(\S+)/;
 		$found = $1;
 		}
-	else { print AUTOTAB $_,"\n"; }
+	else {
+		&print_tempfile(AUTOTAB, $_,"\n");
+		}
 	}
-close(AUTOTAB);
+&close_tempfile(AUTOTAB);
 if ($found) {
 	&system_logged("autofs -r >/dev/null 2>&1 </dev/null");
 	&system_logged("umount $found >/dev/null 2>&1 </dev/null");
@@ -129,41 +137,49 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print FSTAB "$_[2]  $_[1]  $_[3]";
+		&print_tempfile(FSTAB, "$_[2]  $_[1]  $_[3]");
 		local $opts = $_[4] eq "-" ? "" : $_[4];
 		if ($_[6] eq "no") {
 			$opts = join(',' , (split(/,/ , $opts) , "noauto"));
 			}
-		if ($opts eq "") { print FSTAB "  defaults"; }
-		else { print FSTAB "  $opts"; }
-		print FSTAB "  0  ";
-		print FSTAB $_[5] eq "-" ? "0\n" : "$_[5]\n";
+		if ($opts eq "") {
+			&print_tempfile(FSTAB, "  defaults");
+			}
+		else {
+			&print_tempfile(FSTAB, "  $opts");
+			}
+		&print_tempfile(FSTAB, "  0  ");
+		&print_tempfile(FSTAB, $_[5] eq "-" ? "0\n" : "$_[5]\n");
+		}
+	else {
+		&print_tempfile(FSTAB, $_,"\n");
 		}
-	else { print FSTAB $_,"\n"; }
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 local $found;
 open(AUTOTAB, $config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
-open(AUTOTAB, "> $config{autofs_file}");
+&open_tempfile(AUTOTAB, "> $config{autofs_file}");
 foreach (@autotab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print AUTOTAB "$_[1]  $_[2]  ",
-				($_[4] eq "-" ? "" : "-$_[4]"),"\n";
+		&print_tempfile(AUTOTAB, "$_[1]  $_[2]  ",
+				($_[4] eq "-" ? "" : "-$_[4]"),"\n");
 		$found++;
 		}
-	else { print AUTOTAB $_,"\n"; }
+	else {
+		&print_tempfile(AUTOTAB, $_,"\n");
+		}
 	}
-close(AUTOTAB);
+&close_tempfile(AUTOTAB);
 if ($found) {
 	&system_logged("autofs -r >/dev/null 2>&1 </dev/null");
 	}
@@ -624,7 +640,7 @@ if ($_[0] eq "nfs") {
 		}
 
 	# Try a test mount to see if filesystem is available
-	local $temp = &tempname();
+	local $temp = &transname();
 	mkdir($temp, 0755);
 	local $mout = `mount $in{'nfs_host'}:$in{'nfs_dir'} $temp 2>&1`;
 	if ($mout =~ /No such file or directory/i) {
