@@ -62,7 +62,10 @@ while(<FSTAB>) {
 	if ($p[2] eq "proc") { $p[0] = "proc"; }
 	elsif ($p[2] eq "auto") { $p[2] = "*"; }
 	elsif ($p[2] eq "swap") { $p[1] = "swap"; }
-	elsif ($p[2] eq $smbfs_fs || $p[2] eq "cifs") { $p[0] =~ s/\//\\/g; }
+	elsif ($p[2] eq $smbfs_fs || $p[2] eq "cifs") {
+		$p[0] =~ s/\\040/ /g;
+		$p[0] =~ s/\//\\/g;
+		}
 	$rv[$i] = [ $p[1], $p[0], $p[2] ];
 	$rv[$i]->[5] = "yes";
 	@o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
@@ -157,27 +160,32 @@ if ($_[2] eq "auto") {
 	elsif ($amd_support == 2) {
 		# Adding a new automounter mount
 		local @amfs = split(/\s+/, $config{'auto_file'});
-		open(AMD, ">>$amfs[0]");
-		print AMD "\n";
-		print AMD "[ $_[0] ]\n";
-		print AMD "map_name = $_[1]\n";
-		close(AMD);
+		&open_tempfile(AMD, ">>$amfs[0]");
+		&print_tempfile(AMD, "\n");
+		&print_tempfile(AMD, "[ $_[0] ]\n");
+		&print_tempfile(AMD, "map_name = $_[1]\n");
+		&close_tempfile(AMD);
 		}
 	}
 elsif ($_[2] eq "autofs") {
 	# Adding a new automounter mount
-	open(AUTO, ">> $config{'autofs_file'}");
-	print AUTO "$_[0]  $_[1]";
-	if ($_[3]) { print AUTO "  ",&autofs_args($_[3]); }
-	print AUTO "\n";
-	close(AUTO);
+	&open_tempfile(AUTO, ">> $config{'autofs_file'}");
+	&print_tempfile(AUTO, "$_[0]  $_[1]");
+	if ($_[3]) {
+		&print_tempfile(AUTO, "  ",&autofs_args($_[3]));
+		}
+	&print_tempfile(AUTO, "\n");
+	&close_tempfile(AUTO);
 	}
 else {
 	# Adding a normal mount to the fstab file
 	local $dev = $_[1];
-	$dev =~ s/\\/\//g if ($_[2] eq $smbfs_fs || $_[2] eq "cifs");
-	open(FSTAB, ">> $config{fstab_file}");
-	print FSTAB "$dev  $_[0]  $_[2]";
+	if ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
+		$dev =~ s/\\/\//g;
+		$dev =~ s/ /\\040/g;
+		}
+	&open_tempfile(FSTAB, ">> $config{fstab_file}");
+	&print_tempfile(FSTAB, "$dev  $_[0]  $_[2]");
 	local @opts = $_[3] eq "-" ? ( ) : split(/,/, $_[3]);
 	if ($_[5] eq "no") {
 		push(@opts, "noauto");
@@ -188,11 +196,11 @@ else {
 	if ($_[2] eq "bind") {
 		push(@opts, "bind");
 		}
-	if (!@opts) { print FSTAB "  defaults"; }
-	else { print FSTAB "  ".join(",", @opts); }
-	print FSTAB "  0  ";
-	print FSTAB $_[4] eq "-" ? "0\n" : "$_[4]\n";
-	close(FSTAB);
+	if (!@opts) { &print_tempfile(FSTAB, "  defaults"); }
+	else { &print_tempfile(FSTAB, "  ".join(",", @opts)); }
+	&print_tempfile(FSTAB, "  0  ");
+	&print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
+	&close_tempfile(FSTAB);
 	}
 undef(@list_mounts_cache);
 }
@@ -209,12 +217,14 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /\signore\s/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print FSTAB "$_[2]  $_[1]  $_[3]";
+		local $dev = $_[2];
+		$dev =~ s/ /\\040/g;
+		&print_tempfile(FSTAB, "$dev  $_[1]  $_[3]");
 		local @opts = $_[4] eq "-" ? ( ) : split(/,/, $_[4]);
 		if ($_[6] eq "no") {
 			push(@opts, "noauto");
@@ -225,14 +235,14 @@ foreach (@fstab) {
 		if ($_[3] eq "bind") {
 			push(@opts, "bind");
 			}
-		if (!@opts) { print FSTAB "  defaults"; }
-		else { print FSTAB "  ".join(",", @opts); }
-		print FSTAB "  0  ";
-		print FSTAB $_[5] eq "-" ? "0\n" : "$_[5]\n";
+		if (!@opts) { &print_tempfile(FSTAB, "  defaults"); }
+		else { &print_tempfile(FSTAB, "  ".join(",", @opts)); }
+		&print_tempfile(FSTAB, "  0  ");
+		&print_tempfile(FSTAB, $_[5] eq "-" ? "0\n" : "$_[5]\n");
 		}
-	else { print FSTAB $_,"\n"; }
+	else { &print_tempfile(FSTAB, $_,"\n"); }
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 if ($amd_support == 1) {
 	# Update older amd configuration
@@ -276,17 +286,21 @@ if ($autofs_support) {
 	open(AUTO, $config{'autofs_file'});
 	@auto = <AUTO>;
 	close(AUTO);
-	open(AUTO, "> $config{'autofs_file'}");
+	&open_tempfile(AUTO, "> $config{'autofs_file'}");
 	foreach (@auto) {
 		chop; ($line = $_) =~ s/#.*$//g;
 		if ($line =~ /\S/ && $i++ == $_[0]) {
-			print AUTO "$_[1]  $_[2]";
-			if ($_[4]) { print AUTO "  ",&autofs_args($_[4]); }
-			print AUTO "\n";
+			&print_tempfile(AUTO, "$_[1]  $_[2]");
+			if ($_[4]) {
+				&print_tempfile(AUTO, "  ",&autofs_args($_[4]));
+				}
+			&print_tempfile(AUTO, "\n");
+			}
+		else {
+			&print_tempfile(AUTO, $_,"\n");
 			}
-		else { print AUTO $_,"\n"; }
 		}
-	close(AUTO);
+	&close_tempfile(AUTO);
 	}
 undef(@list_mounts_cache);
 }
@@ -303,18 +317,19 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, ">$config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line !~ /\S/ || $line =~ /\signore\s/ || $i++ != $_[0]) {
 		# Don't delete this line
-		print FSTAB $_,"\n";
+		&print_tempfile(FSTAB, $_,"\n");
 		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 if ($amd_support == 1) {
 	# Update older amd configuration
+	local $foundamd = 0;
 	local $amd = &read_amd_conf();
 	if ($amd =~ /MOUNTPTS='(.*)'/) {
 		# found mount points line..
@@ -322,12 +337,13 @@ if ($amd_support == 1) {
 		for($j=0; $j<@mpts; $j+=2) {
 			if ($i++ == $_[0]) {
 				splice(@mpts, $j, 2);
+				$foundamd = 1;
 				}
 			}
 		local $mpts = join(" ", @mpts);
 		$amd =~ s/MOUNTPTS='(.*)'/MOUNTPTS='$mpts'/;
 		}
-	&write_amd_conf($amd);
+	&write_amd_conf($amd) if ($foundamd);
 	}
 elsif ($amd_support == 2) {
 	# Update new amd configuration
@@ -348,7 +364,7 @@ if ($amd_support) {
 	open(AMD, $config{auto_file});
 	@amd = <AMD>;
 	close(AMD);
-	open(AMD, "> $config{auto_file}");
+	&open_tempfile(AMD, ">$config{auto_file}");
 	foreach (@amd) {
 		if (/MOUNTPTS='(.*)'/) {
 			# found mount points line..
@@ -359,11 +375,11 @@ if ($amd_support) {
 					push(@nmpts, $mpts[$j+1]);
 					}
 				}
-			print AMD "MOUNTPTS='".join(' ', @nmpts)."'\n";
+			&print_tempfile(AMD, "MOUNTPTS='".join(' ', @nmpts)."'\n");
 			}
-		else { print AMD $_; }
+		else { &print_tempfile(AMD, $_); }
 		}
-	close(AMD);
+	&close_tempfile(AMD);
 	}
 
 # Update autofs file
@@ -371,15 +387,15 @@ if ($autofs_support) {
 	open(AUTO, $config{'autofs_file'});
 	@auto = <AUTO>;
 	close(AUTO);
-	open(AUTO, "> $config{'autofs_file'}");
+	&open_tempfile(AUTO, ">$config{'autofs_file'}");
 	foreach (@auto) {
 		chop; ($line = $_) =~ s/#.*$//g;
 		if ($line !~ /\S/ || $i++ != $_[0]) {
 			# keep this line
-			print AUTO $_,"\n";
+			&print_tempfile(AUTO, $_,"\n");
 			}
 		}
-	close(AUTO);
+	&close_tempfile(AUTO);
 	}
 undef(@list_mounts_cache);
 }
@@ -475,7 +491,7 @@ while(<SWAPS>) {
 	chop;
 	if (/^(\/\S+)\s+/) {
 		local $sf = $1;
-		if ($sf =~ /^\/dev\/ide\//) {
+		if ($sf =~ /^\/dev\/ide\// || $sf =~ /^\/dev\/mapper\//) {
 			# check fstab for a mount on a device which is a symlink
 			local @st = stat($sf);
 			foreach $m (@mounts) {
@@ -509,9 +525,11 @@ if ($_[2] eq "bind") {
 	push(@opts, "bind");
 	}
 $opts = @opts ? "-o ".quotemeta(join(",", @opts)) : "";
+&parse_options($_[2], $_[3]);
 if ($_[2] eq "swap") {
 	# Use swapon to add the swap space..
-	$out = &backquote_logged("swapon $_[1] 2>&1");
+	local $priarg = $options{'pri'} ne "" ? "-p $options{'pri'}" : "";
+	$out = &backquote_logged("swapon $priarg $_[1] 2>&1");
 	if ($out =~ /Invalid argument/) {
 		# looks like this swap partition isn't ready yet.. set it up
 		$out = &backquote_logged("mkswap $_[1] 2>&1");
@@ -535,13 +553,13 @@ elsif ($_[2] eq "autofs") {
 	}
 elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 	local $shar = $_[1];
-	$shar =~ s/\\/\//g;
+	$shar =~ s/\\/\//g if ($shar =~ /^\\/);
 	local $support = $_[2] eq $smbfs_fs ? $smbfs_support : $cifs_support;
 	return uc($_[2])." not supported" if (!$support);
 	local $qshar = quotemeta($shar);
 	if ($support >= 3) {
 		# SMB filesystem mounted with mount command
-		local $temp = &tempname();
+		local $temp = &transname();
 		local $ex = &system_logged("mount -t $_[2] $opts $qshar $_[0] >$temp 2>&1 </dev/null");
 		local $out = `cat $temp`;
 		unlink($temp);
@@ -552,7 +570,6 @@ elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 		}
 	elsif ($support == 2) {
 		# SMB filesystem mounted with version 2.x smbmount
-		&parse_options($_[2], $_[3]);
 		$opts =
 		    ($options{'user'} ? "-U $options{'user'} " : "").
 		    ($options{'passwd'} ? "" : "-N ").
@@ -584,7 +601,6 @@ elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 		}
 	elsif ($support == 1) {
 		# SMB filesystem mounted with older smbmount
-		&parse_options($_[2], $_[3]);
 		$shortname = &get_system_hostname();
 		if ($shortname =~ /^([^\.]+)\.(.+)$/) { $shortname = $1; }
 		$opts =
@@ -842,12 +858,13 @@ elsif ($_[0] eq "swap") {
 elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 	# Windows filesystem
 	$_[1] =~ /^\\\\(.*)\\(.*)$/;
+	local ($server, $share) = ($1, $2);
 	print "<tr> <td><b>$text{'linux_smbserver'}</b></td>\n";
-	print "<td><input name=smbfs_server value=\"$1\" size=20>\n";
+	print "<td><input name=smbfs_server value=\"$server\" size=20>\n";
 	&smb_server_chooser_button("smbfs_server");
 	print "</td>\n";
 	print "<td><b>$text{'linux_smbshare'}</b></td>\n";
-	print "<td><input name=smbfs_share value=\"$2\" size=20>\n";
+	print "<td><input name=smbfs_share value=\"$share\" size=20>\n";
 	&smb_share_chooser_button("smbfs_server", "smbfs_share");
 	print "</td> </tr>\n";
 	}
@@ -1307,7 +1324,12 @@ elsif ($_[0] eq "autofs") {
 	}
 elsif ($_[0] eq "swap") {
 	# Swap has no options..
-	print "<tr> <td><i>$text{'linux_noopts'}</i></td> </tr>\n";
+	print "<tr> <td width=25%><b>$text{'linux_swappri'}</b></td>\n";
+	print "<td width=25%>",&ui_opt_textbox("swap_pri", $options{'pri'}, 6,
+				     $text{'default'}),"</td>\n";
+
+	print "<td width=50% colspan=2></td>\n";
+	print "</tr>\n";
 	}
 elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 	# SMB filesystems have a few options..
@@ -1325,108 +1347,119 @@ elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 	printf "<td><input type=password name=smbfs_passwd size=15 value=\"%s\"></td> </tr>\n",
 		$support == 4 ? $options{'password'} : $options{'passwd'};
 
-	if ($support != 2) {
-		print "<tr> <td><b>$text{'linux_uid'}</b></td>\n";
-		printf "<td><input name=smbfs_uid size=8 value=\"%s\">\n",
-			defined($options{"uid"}) ? getpwuid($options{"uid"}) : "";
-		print &user_chooser_button("smbfs_uid", 0),"</td>\n";
+	if (!$access{'simopts'}) {
+		if ($support != 2) {
+			print "<tr> <td><b>$text{'linux_uid'}</b></td>\n";
+			printf "<td><input name=smbfs_uid size=8 value=\"%s\">\n",
+				defined($options{"uid"}) ? getpwuid($options{"uid"}) : "";
+			print &user_chooser_button("smbfs_uid", 0),"</td>\n";
 
-		print "<td><b>$text{'linux_gid'}</b></td>\n";
-		printf "<td><input name=smbfs_gid size=8 value=\"%s\">\n",
-			defined($options{"gid"}) ? getgrgid($options{"gid"}) : "";
-		print &group_chooser_button("smbfs_gid", 0),"</td>\n";
-		}
+			print "<td><b>$text{'linux_gid'}</b></td>\n";
+			printf "<td><input name=smbfs_gid size=8 value=\"%s\">\n",
+				defined($options{"gid"}) ? getgrgid($options{"gid"}) : "";
+			print &group_chooser_button("smbfs_gid", 0),"</td>\n";
+			}
 
-	if ($support == 1) {
-		print "<tr> <td><b>$text{'linux_sname'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_sname_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"servername"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_sname_def value=0 %s>\n",
-			defined($options{"servername"}) ? "checked" : "";
-		print "<input size=10 name=smbfs_sname value=\"$options{servername}\"></td>\n";
-		}
-	elsif ($support == 2) {
-		print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"workgroup"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
-			defined($options{"workgroup"}) ? "checked" : "";
-		print "<input size=10 name=smbfs_wg value=\"$options{'workgroup'}\"></td>\n";
-		}
+		if ($support == 1) {
+			print "<tr> <td><b>$text{'linux_sname'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_sname_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"servername"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_sname_def value=0 %s>\n",
+				defined($options{"servername"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_sname value=\"$options{servername}\"></td>\n";
+			}
+		elsif ($support == 2) {
+			print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"workgroup"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
+				defined($options{"workgroup"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_wg value=\"$options{'workgroup'}\"></td>\n";
+			}
 
-	if ($support < 3) {
-		print "<td><b>$text{'linux_cname'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"clientname"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
-			defined($options{"clientname"}) ? "checked" : "";
-		print "<input size=10 name=smbfs_cname value=\"$options{clientname}\"></td> </tr>\n";
+		if ($support < 3) {
+			print "<td><b>$text{'linux_cname'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"clientname"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
+				defined($options{"clientname"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_cname value=\"$options{clientname}\"></td> </tr>\n";
 
-		print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
-		printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
-			defined($options{"machinename"}) ? "" : "checked", $text{'linux_auto'};
-		printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
-			defined($options{"machinename"}) ? "checked" : "";
-		print "<input size=30 name=smbfs_mname value=\"$options{machinename}\"></td> </tr>\n";
-		}
-	
-	if ($support == 1) {
-		print "<tr> <td><b>$text{'linux_fmode'}</b></td>\n";
-		printf
-		    "<td><input name=smbfs_fmode size=5 value=\"%s\"></td>\n",
-		    defined($options{'fmode'}) ? $options{'fmode'} : "755";
+			print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
+			printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
+				defined($options{"machinename"}) ? "" : "checked", $text{'linux_auto'};
+			printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
+				defined($options{"machinename"}) ? "checked" : "";
+			print "<input size=30 name=smbfs_mname value=\"$options{machinename}\"></td> </tr>\n";
+			}
+		
+		if ($support == 1) {
+			print "<tr> <td><b>$text{'linux_fmode'}</b></td>\n";
+			printf
+			    "<td><input name=smbfs_fmode size=5 value=\"%s\"></td>\n",
+			    defined($options{'fmode'}) ? $options{'fmode'} : "755";
 
-		print "<td><b>$text{'linux_dmode'}</b></td>\n";
-		printf
-		    "<td><input name=smbfs_dmode size=5 value=\"%s\"></td>\n",
-		    defined($options{'dmode'}) ? $options{'dmode'} : "755";
-		print "</tr>\n";
-		}
-	elsif ($support >= 3) {
-		print "<tr> <td><b>$text{'linux_fmode'}</b></td> <td>\n";
-		printf"<input type=radio name=smbfs_fmask_def value=1 %s> %s\n",
-			defined($options{'fmask'}) ? "" : "checked",
-			$text{'default'};
-		printf"<input type=radio name=smbfs_fmask_def value=0 %s>\n",
-			defined($options{'fmask'}) ? "checked" : "";
-		printf "<input name=smbfs_fmask size=5 value='%s'></td>\n",
-			$options{'fmask'};
+			print "<td><b>$text{'linux_dmode'}</b></td>\n";
+			printf
+			    "<td><input name=smbfs_dmode size=5 value=\"%s\"></td>\n",
+			    defined($options{'dmode'}) ? $options{'dmode'} : "755";
+			print "</tr>\n";
+			}
+		elsif ($support >= 3) {
+			print "<tr> <td><b>$text{'linux_fmode'}</b></td> <td>\n";
+			printf"<input type=radio name=smbfs_fmask_def value=1 %s> %s\n",
+				defined($options{'fmask'}) ? "" : "checked",
+				$text{'default'};
+			printf"<input type=radio name=smbfs_fmask_def value=0 %s>\n",
+				defined($options{'fmask'}) ? "checked" : "";
+			printf "<input name=smbfs_fmask size=5 value='%s'></td>\n",
+				$options{'fmask'};
 
-		print "<td><b>$text{'linux_dmode'}</b></td> <td>\n";
-		printf"<input type=radio name=smbfs_dmask_def value=1 %s> %s\n",
-			defined($options{'dmask'}) ? "" : "checked",
-			$text{'default'};
-		printf"<input type=radio name=smbfs_dmask_def value=0 %s>\n",
-			defined($options{'dmask'}) ? "checked" : "";
-		printf "<input name=smbfs_dmask size=5 value='%s'></td></tr>\n",
-			$options{'dmask'};
+			print "<td><b>$text{'linux_dmode'}</b></td> <td>\n";
+			printf"<input type=radio name=smbfs_dmask_def value=1 %s> %s\n",
+				defined($options{'dmask'}) ? "" : "checked",
+				$text{'default'};
+			printf"<input type=radio name=smbfs_dmask_def value=0 %s>\n",
+				defined($options{'dmask'}) ? "checked" : "";
+			printf "<input name=smbfs_dmask size=5 value='%s'></td></tr>\n",
+				$options{'dmask'};
 
-		print "<tr> <td><b>$text{'linux_ro'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_ro value=1 %s> $text{'yes'}\n",
-			defined($options{"ro"}) ? "checked" : "";
-		printf "<input type=radio name=smbfs_ro value=0 %s> $text{'no'}</td>\n",
-			defined($options{"ro"}) ? "" : "checked";
+			print "<tr> <td><b>$text{'linux_ro'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_ro value=1 %s> $text{'yes'}\n",
+				defined($options{"ro"}) ? "checked" : "";
+			printf "<input type=radio name=smbfs_ro value=0 %s> $text{'no'}</td>\n",
+				defined($options{"ro"}) ? "" : "checked";
 
-		print "<td><b>$text{'linux_user'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_user2 value=1 %s> $text{'yes'}\n",
-			defined($options{"user"}) ? "checked" : "";
-		printf "<input type=radio name=smbfs_user2 value=0 %s> $text{'no'}</td> </tr>\n",
-			defined($options{"user"}) ? "" : "checked";
-		}
-	if ($support == 4) {
-		print "<tr> <td><b>$text{'linux_cname'}</b></td>\n";
-		printf "<td colspan=3><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"netbiosname"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
-			defined($options{"netbiosname"}) ? "checked" : "";
-		print "<input size=40 name=smbfs_cname value=\"$options{netbiosname}\"></td> </tr>\n";
+			print "<td><b>$text{'linux_user'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_user2 value=1 %s> $text{'yes'}\n",
+				defined($options{"user"}) ? "checked" : "";
+			printf "<input type=radio name=smbfs_user2 value=0 %s> $text{'no'}</td> </tr>\n",
+				defined($options{"user"}) ? "" : "checked";
+			}
+		if ($support == 4) {
+			print "<tr> <td><b>$text{'linux_cname'}</b></td>\n";
+			printf "<td colspan=3><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"netbiosname"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
+				defined($options{"netbiosname"}) ? "checked" : "";
+			print "<input size=40 name=smbfs_cname value=\"$options{netbiosname}\"></td> </tr>\n";
 
-		print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
-		printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
-			defined($options{"ip"}) ? "" : "checked", $text{'linux_auto'};
-		printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
-			defined($options{"ip"}) ? "checked" : "";
-		print "<input size=40 name=smbfs_mname value=\"$options{ip}\"></td> </tr>\n";
+			print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
+			printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
+				defined($options{"ip"}) ? "" : "checked", $text{'linux_auto'};
+			printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
+				defined($options{"ip"}) ? "checked" : "";
+			print "<input size=40 name=smbfs_mname value=\"$options{ip}\"></td> </tr>\n";
+
+			print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"workgroup"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
+				defined($options{"workgroup"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_wg value=\"$options{'workgroup'}\"></td>\n";
+
+			print "</tr>\n";
+			}
 		}
 	}
 elsif ($_[0] eq "reiserfs") {
@@ -1561,7 +1594,7 @@ if (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
 	    }
 
 	# Try a test mount to see if filesystem is available
-	$temp = &tempname();
+	$temp = &transname();
 	mkdir($temp, 0755);
 	$mout = `mount -t $_[0] $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
 	if ($mout =~ /No such file or directory/i) {
@@ -1854,98 +1887,100 @@ elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 		$options{$support == 4 ? 'password' : 'passwd'} = $in{smbfs_passwd};
 		}
 
-	if ($support != 2) {
-		delete($options{uid});
-		if ($in{smbfs_uid} ne "") { $options{uid} = getpwnam($in{smbfs_uid}); }
+	if (!$access{'simopts'}) {
+		if ($support != 2) {
+			delete($options{uid});
+			if ($in{smbfs_uid} ne "") { $options{uid} = getpwnam($in{smbfs_uid}); }
 
-		delete($options{gid});
-		if ($in{smbfs_gid} ne "") { $options{gid} = getgrnam($in{smbfs_gid}); }
-		}
+			delete($options{gid});
+			if ($in{smbfs_gid} ne "") { $options{gid} = getgrnam($in{smbfs_gid}); }
+			}
 
-	if ($support == 1) {
-		delete($options{servername});
-		if (!$in{smbfs_sname_def})
-			{ $options{servername} = $in{smbfs_sname}; }
-		}
-	elsif ($support == 2) {
-		delete($options{workgroup});
-		if (!$in{smbfs_wg_def})
-			{ $options{workgroup} = $in{smbfs_wg}; }
-		}
+		if ($support == 1) {
+			delete($options{servername});
+			if (!$in{smbfs_sname_def})
+				{ $options{servername} = $in{smbfs_sname}; }
+			}
+		elsif ($support == 2 || $support == 4) {
+			delete($options{workgroup});
+			if (!$in{smbfs_wg_def})
+				{ $options{workgroup} = $in{smbfs_wg}; }
+			}
 
-	if ($support < 3) {
-		delete($options{clientname});
-		if (!$in{smbfs_cname_def})
-			{ $options{clientname} = $in{smbfs_cname}; }
+		if ($support < 3) {
+			delete($options{clientname});
+			if (!$in{smbfs_cname_def})
+				{ $options{clientname} = $in{smbfs_cname}; }
 
-		delete($options{machinename});
-		if (!$in{smbfs_mname_def})
-			{ $options{machinename} = $in{smbfs_mname}; }
-		elsif (!gethostbyname($in{'smbfs_server'})) {
-			# No hostname found for the server.. try to guess
-			local($out, $sname);
-			$sname = $in{'smbfs_server'};
-			$out = `$config{'nmblookup_path'} -d 0 $sname 2>&1`;
-			if (!$? && $out =~ /^([0-9\.]+)\s+$sname\n/) {
-				$options{machinename} = $1;
+			delete($options{machinename});
+			if (!$in{smbfs_mname_def})
+				{ $options{machinename} = $in{smbfs_mname}; }
+			elsif (!gethostbyname($in{'smbfs_server'})) {
+				# No hostname found for the server.. try to guess
+				local($out, $sname);
+				$sname = $in{'smbfs_server'};
+				$out = `$config{'nmblookup_path'} -d 0 $sname 2>&1`;
+				if (!$? && $out =~ /^([0-9\.]+)\s+$sname\n/) {
+					$options{machinename} = $1;
+					}
 				}
 			}
-		}
-	elsif ($support == 4) {
-		delete($options{"netbiosname"});
-		if (!$in{"smbfs_cname_def"}) {
-			$in{"smbfs_cname"} =~ /^\S+$/ ||
-				&error($text{'linux_ecname'});
-			$options{"netbiosname"} = $in{"smbfs_cname"};
-			}
-		delete($options{"ip"});
-		if (!$in{"smbfs_mname_def"}) {
-			gethostbyname($in{"smbfs_mname"}) ||
-			    &check_ipaddress($in{"smbfs_mname"}) ||
-				&error($text{'linux_emname'});
-			$options{"ip"} = $in{"smbfs_mname"};
-			}
-		}
-
-	if ($support == 1) {
-		delete($options{fmode});
-		if ($in{smbfs_fmode} !~ /^[0-7]{3}$/) {
-			&error(&text('linux_efmode', $in{'smbfs_fmode'}));
-			}
-		elsif ($in{smbfs_fmode} ne "755")
-			{ $options{fmode} = $in{smbfs_fmode}; }
-
-		delete($options{dmode});
-		if ($in{smbfs_dmode} !~ /^[0-7]{3}$/) {
-			&error(&text('linux_edmode', $in{'smbfs_dmode'}));
-			}
-		elsif ($in{smbfs_dmode} ne "755")
-			{ $options{dmode} = $in{smbfs_dmode}; }
-		}
-	elsif ($support >= 3) {
-		if ($in{'smbfs_fmask_def'}) {
-			delete($options{'fmask'});
-			}
-		else {
-			$in{'smbfs_fmask'} =~ /^[0-7]{3}$/ ||
-			    &error(&text('linux_efmode', $in{'smbfs_fmask'}));
-			$options{'fmask'} = $in{'smbfs_fmask'};
+		elsif ($support == 4) {
+			delete($options{"netbiosname"});
+			if (!$in{"smbfs_cname_def"}) {
+				$in{"smbfs_cname"} =~ /^\S+$/ ||
+					&error($text{'linux_ecname'});
+				$options{"netbiosname"} = $in{"smbfs_cname"};
+				}
+			delete($options{"ip"});
+			if (!$in{"smbfs_mname_def"}) {
+				gethostbyname($in{"smbfs_mname"}) ||
+				    &check_ipaddress($in{"smbfs_mname"}) ||
+					&error($text{'linux_emname'});
+				$options{"ip"} = $in{"smbfs_mname"};
+				}
 			}
 
-		if ($in{'smbfs_dmask_def'}) {
-			delete($options{'dmask'});
-			}
-		else {
-			$in{'smbfs_dmask'} =~ /^[0-7]{3}$/ ||
-			    &error(&text('linux_edmode', $in{'smbfs_dmask'}));
-			$options{'dmask'} = $in{'smbfs_dmask'};
-			}
+		if ($support == 1) {
+			delete($options{fmode});
+			if ($in{smbfs_fmode} !~ /^[0-7]{3}$/) {
+				&error(&text('linux_efmode', $in{'smbfs_fmode'}));
+				}
+			elsif ($in{smbfs_fmode} ne "755")
+				{ $options{fmode} = $in{smbfs_fmode}; }
 
-		delete($options{'ro'}); delete($options{'rw'});
-		if ($in{'smbfs_ro'}) { $options{'ro'} = ''; }
+			delete($options{dmode});
+			if ($in{smbfs_dmode} !~ /^[0-7]{3}$/) {
+				&error(&text('linux_edmode', $in{'smbfs_dmode'}));
+				}
+			elsif ($in{smbfs_dmode} ne "755")
+				{ $options{dmode} = $in{smbfs_dmode}; }
+			}
+		elsif ($support >= 3) {
+			if ($in{'smbfs_fmask_def'}) {
+				delete($options{'fmask'});
+				}
+			else {
+				$in{'smbfs_fmask'} =~ /^[0-7]{3}$/ ||
+				    &error(&text('linux_efmode', $in{'smbfs_fmask'}));
+				$options{'fmask'} = $in{'smbfs_fmask'};
+				}
 
-		delete($options{'user'});
-		if ($in{'smbfs_user2'}) { $options{'user'} = ''; }
+			if ($in{'smbfs_dmask_def'}) {
+				delete($options{'dmask'});
+				}
+			else {
+				$in{'smbfs_dmask'} =~ /^[0-7]{3}$/ ||
+				    &error(&text('linux_edmode', $in{'smbfs_dmask'}));
+				$options{'dmask'} = $in{'smbfs_dmask'};
+				}
+
+			delete($options{'ro'}); delete($options{'rw'});
+			if ($in{'smbfs_ro'}) { $options{'ro'} = ''; }
+
+			delete($options{'user'});
+			if ($in{'smbfs_user2'}) { $options{'user'} = ''; }
+			}
 		}
 	}
 elsif ($_[0] eq "reiserfs") {
@@ -2017,6 +2052,17 @@ elsif ($_[0] eq "ntfs") {
 	if ($in{ntfs_gid} ne "")
 		{ $options{"gid"} = getgrnam($in{ntfs_gid}); }
 	}
+elsif ($_[0] eq "swap") {
+	# Save SWAP options
+	if ($in{'swap_pri_def'}) {
+		delete($options{'pri'});
+		}
+	else {
+		$in{'swap_pri'} =~ /^\d+$/ && $in{'swap_pri'} <= 32767 ||
+			&error($text{'linux_eswappri'});
+		$options{'pri'} = $in{'swap_pri'};
+		}
+	}
 
 # Add loop option if mounting a normal file
 if ($_[0] ne "swap" && $_[0] ne "auto" && $_[0] ne "autofs" &&
@@ -2059,11 +2105,11 @@ close(SMBOPTS);
 sub write_smbopts
 {
 local($_);
-open(SMBOPTS, "> $module_config_directory/smbfs");
+&open_tempfile(SMBOPTS, "> $module_config_directory/smbfs");
 foreach (keys %smbopts) {
-	print SMBOPTS "$_\t$smbopts{$_}\n";
+	&print_tempfile(SMBOPTS, "$_\t$smbopts{$_}\n");
 	}
-close(SMBOPTS);
+&close_tempfile(SMBOPTS);
 }
 
 
@@ -2072,7 +2118,8 @@ close(SMBOPTS);
 sub create_swap
 {
 local($out, $bl);
-$bl = $_[1] * ($_[2] eq "m" ? 1024 : 1);
+$bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
+	       $_[2] eq "m" ? 1024 : 1);
 $out = `dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1`;
 if ($?) { return "dd failed : $out"; }
 $out = `mkswap $_[0] $bl 2>&1`;
@@ -2176,9 +2223,10 @@ return $rv;
 # write_amd_conf(text)
 sub write_amd_conf
 {
-open(AMD, ">$config{'auto_file'}");
-print AMD $_[0];
-close(AMD);
+local @af = split(/\s+/, $config{'auto_file'});
+&open_tempfile(AMD, ">$config{'auto_file'}");
+&print_tempfile(AMD, $_[0]);
+&close_tempfile(AMD);
 }
 
 # parse_amd_conf()
