@@ -4,8 +4,10 @@
 
 do '../web-lib.pl';
 &init_config();
-require '../ui-lib.pl';
+do '../ui-lib.pl';
 %access = &get_module_acl();
+$filesystem_users_file = "$module_config_directory/filesystem-users";
+@access_fs = split(/\s+/, $access{'fs'});
 
 # get_mount(directory|'swap', device)
 # Returns the index of this mount, or -1 if not known
@@ -90,8 +92,7 @@ return @rv ? join(",", @rv) : "-";
 # to a creation program, and then redirected back to the original mount cgi
 sub swap_form
 {
-&header("Create Swap File", "");
-print "<hr>\n";
+&ui_print_header(undef, "Create Swap File", "");
 print "<form action=create_swap.cgi>\n";
 foreach $k (keys %in) {
 	print "<input type=hidden name=\"$k\" value=\"$in{$k}\">\n";
@@ -101,11 +102,11 @@ print "The swap file <tt>$_[0]</tt> does not exist.<p>\n";
 print "Create and mount a swap file with size <input name=cswap_size size=6>\n";
 print "<select name=cswap_units>\n";
 print "<option value=k> kB\n";
-print "<option value=m> MB\n";
+print "<option value=m selected> MB\n";
+print "<option value=g> GB\n";
 print "</select>\n";
 print "<input type=submit value=\"Create\"></form>\n";
-print "<hr>\n";
-&footer("", "mount list");
+&ui_print_footer("", "mount list");
 exit;
 }
 
@@ -114,7 +115,9 @@ sub nfs_server_chooser_button
 {
 local($form);
 $form = @_ > 1 ? $_[1] : 0;
-print "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"../$module_name/nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
+if ($access{'browse'}) {
+	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"../$module_name/nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
+	}
 }
 
 # nfs_export_chooser_button(serverinput, exportinput, [form])
@@ -122,7 +125,9 @@ sub nfs_export_chooser_button
 {
 local($form);
 $form = @_ > 2 ? $_[2] : 0;
-print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"../$module_name/nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
+if ($access{'browse'}) {
+	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"../$module_name/nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
+	}
 }
 
 # smb_server_chooser_button(serverinput, [form])
@@ -130,7 +135,7 @@ sub smb_server_chooser_button
 {
 local($form);
 $form = @_ > 1 ? $_[1] : 0;
-if (&has_command($config{'smbclient_path'})) {
+if (&has_command($config{'smbclient_path'}) && $access{'browse'}) {
 	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; smb_server = window.open(\"../$module_name/smb_server.cgi\", \"smb_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
 	}
 }
@@ -140,7 +145,7 @@ sub smb_share_chooser_button
 {
 local($form);
 $form = @_ > 2 ? $_[2] : 0;
-if (&has_command($config{'smbclient_path'})) {
+if (&has_command($config{'smbclient_path'}) && $access{'browse'}) {
 	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; smb_share = window.open(\"../$module_name/smb_share.cgi?server=\"+document.forms[$form].$_[0].value, \"smb_share\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_share.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
 	}
 }
@@ -153,18 +158,31 @@ else {
 	do "$gconfig{'os_type'}-lib.pl";
 	}
 
-# can_edit_fs(dir, device, type, options)
+# can_edit_fs(dir, device, type, options, [is-new])
 # Returns 1 if a filesystem can be edited, 0 otherwise
 sub can_edit_fs
 {
-return 1 if (!$access{'fs'});
-local @dirs = split(/\s+/, $access{'fs'});
-local $d;
-foreach $d (@dirs) {
-	return 1 if ($d eq "/" || $d eq $_[0] ||
-		     substr($_[0], 0, length($d)+1) eq "$d/");
+local $ok = 1;
+if (@access_fs) {
+	local $d;
+	foreach $d (@access_fs) {
+		$ok = 0 if (!&is_under_directory($d, $_[0]));
+		}
 	}
-return 0;
+$ok = 0 if (!&can_fstype($_[2]));
+if ($access{'user'} && !$_[4]) {
+	local $users = &get_filesystem_users();
+	$ok = 0 if ($users->{$_[0]} ne $remote_user);
+	}
+return $ok;
+}
+
+# can_fstype(type)
+sub can_fstype
+{
+return 1 if (!$access{'types'});
+local @types = split(/\s+/, $access{'types'});
+return &indexof($_[0], @types) >= 0;
 }
 
 # compile_program(name, default-arch)
@@ -173,17 +191,17 @@ return 0;
 sub compile_program
 {
 return if (-r "$module_config_directory/$_[0]" &&
-	   system("$module_config_directory/$_[0] >/dev/null 2>&1 </dev/null")
-		== 0);
+   system("$module_config_directory/$_[0] >/dev/null 2>&1 </dev/null")
+	== 0);
 local $arch = `uname -m`;
 $arch =~ s/\r|\n//g;
 local $re = $_[1];
 if ($re && $arch =~ /^$re$/i && -r "$module_root_directory/$_[0]" &&
-    system("$module_root_directory/$_[0] >/dev/null 2>&1 </dev/null") == 0) {
-	# Compiled program for this architecture already exists and is working,
-	# so can just copy
-	system("cp $module_root_directory/$_[0] $module_config_directory/$_[0]");
-	}
+system("$module_root_directory/$_[0] >/dev/null 2>&1 </dev/null") == 0) {
+# Compiled program for this architecture already exists and is working,
+# so can just copy
+system("cp $module_root_directory/$_[0] $module_config_directory/$_[0]");
+}
 else {
 	# Need to compile
 	local ($cc) = (&has_command("gcc") || &has_command("cc"));
@@ -195,5 +213,46 @@ else {
 	}
 }
 
+# get_filesystem_users()
+# Returns a mapping between filesystems and their owners
+sub get_filesystem_users
+{
+local %users;
+&read_file($filesystem_users_file, \%users);
+return \%users;
+}
+
+# save_filesystem_users(&usermap)
+# Saves the filesystem owner mapping
+sub save_filesystem_users
+{
+&write_file($filesystem_users_file, $_[0]);
+}
+
+# can_delete_directory(mount-point)
+# Returns 1 if some directory should be deleted when un-mounting
+sub can_delete_directory
+{
+local @dirs = split(/\s+/, $config{'delete_under'});
+return 0 if (!@dirs);
+return 0 if ($_[0] eq "swap");
+local $d;
+foreach $d (@dirs) {
+	return 1 if (&is_under_directory($d, $_[0]));
+	}
+return 0;
+}
+
+# delete_unmounted(dir, device)
+# If some directory is no longer in the permanent mount list, delete it if it
+# is under the list of dirs to auto-delete
+sub delete_unmounted
+{
+if (&can_delete_directory($_[0]) &&
+    &get_mount($_[0], $_[1]) < 0) {
+	&system_logged("rmdir ".quotemeta($_[0]));
+	}
+}
+
 1;
 
