@@ -9,7 +9,7 @@ char *expand_flags(int f);
 
 int main(void)
 {
-struct statvfs *mntlist;
+struct statfs *mntlist;
 int n, i;
 
 n = getmntinfo(&mntlist, MNT_NOWAIT);
