@@ -45,17 +45,17 @@ sub create_mount
 local(@mlist, @amd, $_); local($opts);
 
 # Adding a normal mount to the fstab file
-open(FSTAB, ">> $config{'fstab_file'}");
-print FSTAB "$_[1]  $_[0]  $_[2]";
+&open_tempfile(FSTAB, ">> $config{'fstab_file'}");
+&print_tempfile(FSTAB, "$_[1]  $_[0]  $_[2]");
 $opts = $_[3] eq "-" ? "" : $_[3];
 if ($_[5] eq "no") {
 	$opts = join(',' , (split(/,/ , $opts) , "noauto"));
 	}
-if ($opts eq "") { print FSTAB "  defaults"; }
-else { print FSTAB "  $opts"; }
-print FSTAB "  0  ";
-print FSTAB $_[4] eq "-" ? "0\n" : "$_[4]\n";
-close(FSTAB);
+if ($opts eq "") { &print_tempfile(FSTAB, "  defaults"); }
+else { &print_tempfile(FSTAB, "  $opts"); }
+&print_tempfile(FSTAB, "  0  ");
+&print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
+&close_tempfile(FSTAB);
 }
 
 
@@ -70,24 +70,26 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print FSTAB "$_[2]  $_[1]  $_[3]";
+		&print_tempfile(FSTAB, "$_[2]  $_[1]  $_[3]");
 		$opts = $_[4] eq "-" ? "" : $_[4];
 		if ($_[6] eq "no") {
 			$opts = join(',' , (split(/,/ , $opts) , "noauto"));
 			}
-		if ($opts eq "") { print FSTAB "  defaults"; }
-		else { print FSTAB "  $opts"; }
-		print FSTAB "  0  ";
-		print FSTAB $_[5] eq "-" ? "0\n" : "$_[5]\n";
+		if ($opts eq "") { &print_tempfile(FSTAB, "  defaults"); }
+		else { &print_tempfile(FSTAB, "  $opts"); }
+		&print_tempfile(FSTAB, "  0  ");
+		&print_tempfile(FSTAB, $_[5] eq "-" ? "0\n" : "$_[5]\n");
+		}
+	else {
+		&print_tempfile(FSTAB, $_,"\n");
 		}
-	else { print FSTAB $_,"\n"; }
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 }
 
 
@@ -102,15 +104,15 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line !~ /\S/ || $i++ != $_[0]) {
 		# Don't delete this line
-		print FSTAB $_,"\n";
+		&print_tempfile(FSTAB, $_,"\n");
 		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 }
 
 
@@ -541,7 +543,7 @@ if ($_[0] eq "nfs") {
 		}
 
 	# Try a test mount to see if filesystem is available
-	$temp = &tempname();
+	$temp = &transname();
 	mkdir($temp, 0755);
 	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
 	if ($mout =~ /No such file or directory/) {
@@ -715,7 +717,8 @@ return @rv ? join(',' , @rv) : "-";
 sub create_swap
 {
 local($out, $bl);
-$bl = $_[1] * ($_[2] eq "m" ? 1024 : 1);
+$bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
+	       $_[2] eq "m" ? 1024 : 1);
 $out = `dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1`;
 if ($?) { return "dd failed : $out"; }
 $out = `mkswap $_[0] $bl 2>&1`;
