@@ -33,22 +33,22 @@ local($len, @mlist, $fcsk, $dir);
 if ($_[2] eq "autofs") {
 	# An autofs mount.. add to /etc/auto_master
 	$len = grep { $_->[2] eq "autofs" } (&list_mounts());
-	open(AUTOTAB, ">> $config{autofs_file}");
-	print AUTOTAB "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n";
-	close(AUTOTAB);
+	&open_tempfile(AUTOTAB, ">> $config{autofs_file}");
+	&print_tempfile(AUTOTAB, "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n");
+	&close_tempfile(AUTOTAB);
 	}
 else {
 	# Add to the fstab file
 	$len = grep { $_->[2] ne "autofs" } (&list_mounts());
-	open(FSTAB, ">> $config{fstab_file}");
+	&open_tempfile(FSTAB, ">> $config{fstab_file}");
 	if ($_[2] eq "ufs" || $_[2] eq "s5fs") {
 		($fsck = $_[1]) =~ s/\/dsk\//\/rdsk\//g;
 		}
 	else { $fsck = "-"; }
 	if ($_[2] eq "swap") { $dir = "-"; }
 	else { $dir = $_[0]; }
-	print FSTAB "$_[1]  $fsck  $dir  $_[2]  $_[4]  $_[5]  $_[3]\n";
-	close(FSTAB);
+	&print_tempfile(FSTAB, "$_[1]  $fsck  $dir  $_[2]  $_[4]  $_[5]  $_[3]\n");
+	&close_tempfile(FSTAB);
 	}
 return $len;
 }
@@ -64,28 +64,28 @@ open(FSTAB, $config{fstab_file});
 close(FSTAB);
 $i = 0;
 
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# found the line not to include
 		}
-	else { print FSTAB $_,"\n"; }
+	else { &print_tempfile(FSTAB, $_,"\n"); }
 	}
 close(FSTAB);
 
 open(AUTOTAB, $config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
-open(AUTOTAB, "> $config{autofs_file}");
+&open_tempfile(AUTOTAB, "> $config{autofs_file}");
 foreach (@autotab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
 		# found line not to include..
 		}
-	else { print AUTOTAB $_,"\n"; }
+	else { &print_tempfile(AUTOTAB, $_,"\n"); }
 	}
-close(AUTOTAB);
+&close_tempfile(AUTOTAB);
 }
 
 
@@ -99,7 +99,7 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
@@ -110,26 +110,26 @@ foreach (@fstab) {
 		else { $fsck = "-"; }
 		if ($_[3] eq "swap") { $dir = "-"; }
 		else { $dir = $_[1]; }
-		print FSTAB "$_[2]  $fsck  $dir  $_[3]  $_[5]  $_[6]  $_[4]\n";
+		&print_tempfile(FSTAB, "$_[2]  $fsck  $dir  $_[3]  $_[5]  $_[6]  $_[4]\n");
 		}
-	else { print FSTAB $_,"\n"; }
+	else { &print_tempfile(FSTAB, $_,"\n"); }
 	}
 close(FSTAB);
 
 open(AUTOTAB, $config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
-open(AUTOTAB, "> $config{autofs_file}");
+&open_tempfile(AUTOTAB, "> $config{autofs_file}");
 foreach (@autotab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print AUTOTAB "$_[1]  $_[2]  ",
-				($_[4] eq "-" ? "" : "-$_[4]"),"\n";
+		&print_tempfile(AUTOTAB, "$_[1]  $_[2]  ",
+				($_[4] eq "-" ? "" : "-$_[4]"),"\n");
 		}
-	else { print AUTOTAB $_,"\n"; }
+	else { &print_tempfile(AUTOTAB, $_,"\n"); }
 	}
-close(AUTOTAB);
+&close_tempfile(AUTOTAB);
 }
 
 
@@ -205,7 +205,7 @@ else {
 		 (defined($options{'readwrite'}) ? "-S " : "").
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
-		local $rtemp = &tempname();
+		local $rtemp = &transname();
 		$rv = system("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
 		$out = `cat $rtemp`; unlink($rtemp);
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
@@ -967,7 +967,7 @@ if ($_[0] eq "nfs") {
 		       "available directories on $in{nfs_host} are:".
 		       "<pre>$dirlist</pre>");
 		}
-	$temp = &tempname();
+	$temp = &transname();
 	mkdir($temp, 0755);
 	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
 	if ($mout =~ /No such file or directory/) {
