@@ -348,28 +348,29 @@ if (@_ > 1 && !$one && $remote_user && !$notabs) {
                 $inlist = "true";
                } 
             }
+        local $catdesc = $text{'longcategory_'.$c};
         if ($in{'cat'} eq $c && $theme_index_page) {
            if ($inlist eq "true") {
 
               if ($c eq "") {
-                print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/others.jpg" width="43" height="44"><br>~;
+                print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/others.jpg" width="43" height="44" title="$catdesc"><br>~;
             &chop_font;
 
                           print qq~</center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~; 
               } elsif ($c eq "webmin") {
                if (@_ > 1) {
-               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0><br>~;
+               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0 title="$catdesc"><br>~;
             &chop_font;
                           print qq~</a></center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~;
                 } else {
-               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0><br>~;            &chop_font;
+               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0 title="$catdesc"><br>~;            &chop_font;
                           print qq~</center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~;
                 }
                } else {
-               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44"><br>~;
+               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -378,7 +379,7 @@ if (@_ > 1 && !$one && $remote_user && !$notabs) {
               }
 
         } else {
-            print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/unknown.jpg" width="43" height="44"><br>~;
+            print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/unknown.jpg" width="43" height="44" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -389,14 +390,14 @@ if (@_ > 1 && !$one && $remote_user && !$notabs) {
         else {
             if ($inlist eq "true") {
               if ($c eq "") {
-                print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/others.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+                print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/others.jpg" width="43" height="44" border=0 alt="$t" title="$catdesc"><br>~;
 
             &chop_font;
 
                 print qq~</a></center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~; 
               } else {
-               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -404,7 +405,7 @@ if (@_ > 1 && !$one && $remote_user && !$notabs) {
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~;
               }
         } else {
-            print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/unknown.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+            print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/unknown.jpg" width="43" height="44" border=0 alt="$t" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -498,11 +499,11 @@ print qq~
 	    # Show page title in tab
 	    local $title = $_[0];
 	    print "<p><table border=0 cellpadding=0 cellspacing=0 width=95% align=center><tr><td><table border=0 cellpadding=0 cellspacing=0 height=20><tr>\n";
-	    print "<td bgcolor=#bae3ff>",
+	    print "<td bgcolor=#bae3ff valign=top>",
 	      "<img src=$gconfig{'webprefix'}/images/tabs/blue_left.jpg width=13 height=22 ",
 	      "alt=\"\">","</td>\n";
 	    print "<td bgcolor=#bae3ff>&nbsp;<b>$title</b>&nbsp;</td>\n";
-	    print "<td bgcolor=#bae3ff>",
+	    print "<td bgcolor=#bae3ff valign=top>",
 	      "<img src=$gconfig{'webprefix'}/images/tabs/blue_right.jpg width=19 height=22 ",
 	      "alt=\"\">","</td>\n";
 	    if ($_[9]) {
@@ -660,7 +661,7 @@ sub theme_error
 &header($text{'error'}, "");
 print "<br>\n";
 print "<h3>",($main::whatfailed ? "$main::whatfailed : " : ""),@_,"</h3>\n";
-&footer();
+&footer($ENV{'HTTP_REFERER'}, $text{'error_previous'});
 }
 
 sub theme_ui_print_unbuffered_header
@@ -671,5 +672,12 @@ $main::printed_unbuffered_header = 1;
 &ui_print_header(@_);
 }
 
+# theme_ui_subheading(text, ...)
+# Returns HTML for a section heading
+sub theme_ui_subheading
+{
+return "<font size=+1>",join("", @_),"</font><br>\n";
+}
+
 1;
 
