@@ -25,7 +25,7 @@ $routes_active_now = 1; #Changes made to routes are always applied
 sub active_interfaces
 {
     local(@rv, @lines, $line);
-    open(IFC, "ipconfig /all |");
+    &open_execute_command(IFC, "ipconfig /all", 1);
     while (<IFC>) {
 	s/\r|\n//g;
 	push(@lines, $_);
@@ -148,7 +148,7 @@ sub boot_interfaces
 #		'up' => 1,
 #		'edit' => 0 });
     my (@lines, $l);
-    open(IFC, "netsh interface ip dump |");
+    &open_execute_command(IFC, "netsh interface ip dump", 1);
     while (<IFC>) {
 	s/\r|\n//g;
 	push(@lines, $_);
@@ -209,7 +209,7 @@ sub save_interface
     } else {
 	$cmd .= "static addr = $ifc->{'address'} mask = $ifc->{'netmask'}";
     }
-    system_logged("$cmd > $logfile 2>&1");
+    system_logged("$cmd >$logfile 2>&1");
 }
 
 # delete_interface(&details)
@@ -261,7 +261,7 @@ sub get_dns_config
     my @lines = ();
     my $dns = {'domain' => []};
     my $i = 0;
-    if (open(CMD, "ipconfig /all |")) {
+    if (&open_execute_command(CMD, "ipconfig /all", 1)) {
 	my $doing_domain = 0;
 	while (<CMD>) {
 	    s/[\n\r]//g;
@@ -283,7 +283,7 @@ sub get_dns_config
 	}
 	close(CMD);
     }
-    if (open(CMD, "netsh interface ip show dns |")) {
+    if (&open_execute_command(CMD, "netsh interface ip show dns", 1)) {
 	my $doing_nameserver = 0;
 	my $i = -1;
 	my $key = "nameserver";
