@@ -13,7 +13,7 @@ do 'linux-lib.pl';
 
 # boot_interfaces()
 # Returns a list of interfaces brought up at boot time
-sub boot_interfaces()
+sub boot_interfaces
 {
 my @ifaces = &get_interface_defs();
 my @autos = &get_auto_defs();
@@ -141,128 +141,6 @@ sub valid_boot_address
 return &check_ipaddress($_[0]);
 }
 
-# get_dns_config()
-# Returns a hashtable containing keys nameserver, domain, search & order
-sub get_dns_config
-{
-local $dns;
-open(RESOLV, "/etc/resolv.conf");
-while(<RESOLV>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/nameserver\s+(.*)/) {
-		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
-		}
-	elsif (/domain\s+(\S+)/) {
-		$dns->{'domain'} = [ $1 ];
-		}
-	elsif (/search\s+(.*)/) {
-		$dns->{'domain'} = [ split(/\s+/, $1) ];
-		}
-	}
-close(RESOLV);
-open(SWITCH, "/etc/nsswitch.conf");
-while(<SWITCH>) {
-	s/\r|\n//g;
-	if (/^\s*hosts:\s+(.*)/) {
-		$dns->{'order'} = $1;
-		}
-	}
-close(SWITCH);
-$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
-return $dns;
-}
-
-# save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
-sub save_dns_config
-{
-&lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
-local @resolv = <RESOLV>;
-close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
-foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
-	}
-if ($_[0]->{'domain'}) {
-	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
-		}
-	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
-		}
-	}
-foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
-	}
-close(RESOLV);
-&unlock_file("/etc/resolv.conf");
-
-&lock_file("/etc/nsswitch.conf");
-open(SWITCH, "/etc/nsswitch.conf");
-local @switch = <SWITCH>;
-close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
-foreach (@switch) {
-	if (/^\s*hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
-		}
-	else { print SWITCH $_; }
-	}
-close(SWITCH);
-&unlock_file("/etc/nsswitch.conf");
-}
-
-$max_dns_servers = 3;
-
-# order_input(&dns)
-# Returns HTML for selecting the name resolution order
-sub order_input
-{
-if ($_[0]->{'order'} =~ /\[/) {
-	# Using a complex resolve list
-	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
-	}
-else {
-	# Can select by menus
-	local @o = split(/\s+/, $_[0]->{'order'});
-	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
-	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus", "db" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "DB" );
-	for($i=1; $i<@srcs; $i++) {
-		local $ii = $i-1;
-		$rv .= "<select name=order_$ii>\n";
-		for($j=0; $j<@srcs; $j++) {
-			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-					$srcs[$j],
-					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j] ? $srcn[$j] : "&nbsp;";
-			}
-		$rv .= "</select>\n";
-		}
-	return $rv;
-	}
-}
-
-# parse_order(&dns)
-# Parses the form created by order_input()
-sub parse_order
-{
-if (defined($in{'order'})) {
-	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
-	$_[0]->{'order'} = $in{'order'};
-	}
-else {
-	local($i, @order);
-	for($i=0; defined($in{"order_$i"}); $i++) {
-		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-		}
-	$_[0]->{'order'} = join(" ", @order);
-	}
-}
-
 # get_hostname()
 sub get_hostname
 {
@@ -276,41 +154,27 @@ local (%conf, $f);
 &system_logged("hostname $_[0] >/dev/null 2>&1");
 foreach $f ("/etc/hostname", "/etc/HOSTNAME") {
 	if (-r $f) {
-		&lock_file($f);
-		open(HOST, ">$f");
-		print HOST $_[0],"\n";
-		close(HOST);
-		&unlock_file($f);
+		&open_lock_tempfile(HOST, ">$f");
+		&print_tempfile(HOST, $_[0],"\n");
+		&close_tempfile(HOST);
 		}
 	}
-&lock_file($network_config);
-&read_env_file($network_config, \%conf);
-$conf{'HOSTNAME'} = $_[0];
-&write_env_file($network_config, \%conf);
-&unlock_file($network_config);
 }
 
 # get_domainname()
 sub get_domainname
 {
-local $d = `domainname`;
+local $d;
+&execute_command("domainname", undef, \$d, undef);
 chop($d);
-return $d eq "(none)" ? "" : $d;
+return $d;
 }
 
 # save_domainname(domain)
 sub save_domainname
 {
 local %conf;
-system("domainname \"$_[0]\" >/dev/null 2>&1");
-&read_env_file($network_config, \%conf);
-if ($_[0]) {
-	$conf{'NISDOMAIN'} = $_[0];
-	}
-else {
-	delete($conf{'NISDOMAIN'});
-	}
-&write_env_file($network_config, \%conf);
+&execute_command("domainname ".quotemeta($_[0]));
 }
 
 sub routing_config_files
@@ -460,11 +324,11 @@ foreach $iface (@ifaces) {
 #    addrfam       the address family (e.g. inet, inet6)
 #    method        the address activation method (e.g. static, dhcp, loopback)
 #    options       is a list of (param, value) pairs
-sub get_interface_defs()
+sub get_interface_defs
 {
 local *CFGFILE;
 my @ret;
-open(CFGFILE, "< $network_interfaces_config") ||
+&open_readfile(CFGFILE, $network_interfaces_config) ||
 	error("Unable to open $network_interfaces_config");
 # read the file line by line
 $line = <CFGFILE>;
@@ -530,7 +394,7 @@ return @ret;
 sub get_auto_defs
 {
 local @rv;
-open(CFGFILE, $network_interfaces_config);
+&open_readfile(CFGFILE, $network_interfaces_config);
 while(<CFGFILE>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//g;
@@ -574,16 +438,16 @@ if (!$found) {
 # the parameters should be (name, addrfam, method, options, mode)
 # with options being an array of (param, value) pairs
 # and mode being 0 for modify and 1 for delete
-sub modify_interface_def($$$$$)
+sub modify_interface_def
 {
 my ($name, $addrfam, $method, $options, $mode) = @_;
 # make a backup copy
 copy("$network_interfaces_config", "$network_interfaces_config~");
 local *OLDCFGFILE, *NEWCFGFILE;
-open(OLDCFGFILE, "< $network_interfaces_config~") ||
+&open_readfile(OLDCFGFILE, "$network_interfaces_config~") ||
 	error("Unable to open $network_interfaces_config");
 &lock_file($network_interfaces_config);
-open(NEWCFGFILE, "> $network_interfaces_config") ||
+&open_tempfile(NEWCFGFILE, "> $network_interfaces_config", 1) ||
 	error("Unable to open $network_interfaces_config");
 
 my $inside_modify_region = 0;
@@ -602,16 +466,16 @@ while (defined ($line=<OLDCFGFILE>)) {
 		}
 	# preserve comments
 	if ($line =~ /^\s*#/) {
-		print NEWCFGFILE $line;
+		&print_tempfile(NEWCFGFILE, $line);
 		}
 	# inside modify region or not ?
 	elsif ($inside_modify_region == 0) {
-               print NEWCFGFILE $line;
+               &print_tempfile(NEWCFGFILE, $line);
 		}
 	else {
 		# should the iface line be changed or the options ?
 		if ($iface_line == 1 && $mode == 0) {
-			print NEWCFGFILE "iface $name $addrfam $method\n";
+			&print_tempfile(NEWCFGFILE, "iface $name $addrfam $method\n");
 			}
                # only write the new options and skip the old ones or just do
                # nothing if mode is delete
@@ -620,7 +484,7 @@ while (defined ($line=<OLDCFGFILE>)) {
                        $new_options_wrote = 1;
                        foreach $option (@$options) {
                                my ($param, $value) = @$option;
-                               print NEWCFGFILE "\t$param $value\n";
+                               &print_tempfile(NEWCFGFILE, "\t$param $value\n");
 				}
 			}
 		}
@@ -628,7 +492,7 @@ while (defined ($line=<OLDCFGFILE>)) {
 	}
 
 close(OLDCFGFILE);
-close(NEWCFGFILE);
+&close_tempfile(NEWCFGFILE);
 &unlock_file($network_interfaces_config);
 }
 
@@ -636,27 +500,25 @@ close(NEWCFGFILE);
 # the parameters should be (name, addrfam, method, options)
 # with options being an array of (param, value) pairs
 # the selection key is (name, addrfam)
-sub new_interface_def($$$$)
+sub new_interface_def
 {
 # make a backup copy
 copy("$network_interfaces_config", "$network_interfaces_config~");
 local *CFGFILE;
-&lock_file($network_interfaces_config);
-open(CFGFILE, ">> $network_interfaces_config") ||
+&open_lock_tempfile(CFGFILE, ">> $network_interfaces_config") ||
 	error("Unable to open $network_interfaces_config");
 local ($name, $addrfam, $method, $options) = @_;
-print CFGFILE "\niface $name $addrfam $method\n";
+&print_tempfile(CFGFILE, "\niface $name $addrfam $method\n");
 foreach $option (@$options) {
 	my ($param, $value) = @$option;
-	print CFGFILE "\t$param $value\n";
+	&print_tempfile(CFGFILE, "\t$param $value\n");
 	}
-close(CFGFILE);
-&unlock_file($network_interfaces_config);
+&close_tempfile(CFGFILE);
 }
 
 # delete an already defined interface
 # the parameters should be (name, addrfam)
-sub delete_interface_def($$$)
+sub delete_interface_def
 {
 	local ($name, $addrfam, $method) = @_;
 	modify_interface_def($name, $addrfam, '', [], 1);
