@@ -143,11 +143,12 @@ if(($_[0]->{'up'} == 1) && (!-e $boot_scripts . "/net." . $_[0]->{'name'})) {
 	## [Gentoo] Some time we'll need to create a start script first.
 	if(!-e $init_scripts . "/net." . $_[0]->{'name'}) {
 		&lock_file($init_scripts . "/net." . $_[0]->{'name'});
-		open(IN, $net_template);
-		open(OUT, ">" . $init_scripts . "/net." . $_[0]->{'name'});
-			&copydata(\*IN, \*OUT);
+		&open_readfile(IN, $net_template);
+		local $tofile = $init_scripts."/net.".$_[0]->{'name'};
+		&open_tempfile(OUT, ">$tofile");
+		&copydata(\*IN, \*OUT) || &error(&text("efilewrite", $tofile, $!));
 		close IN;
-		close OUT;
+		&close_tempfile(OUT);
 		&unlock_file($init_scripts . "/net." . $_[0]->{'name'});
 		chmod oct("0755"), $init_scripts . "/net." . $_[0]->{'name'};
 	}
@@ -156,7 +157,7 @@ if(($_[0]->{'up'} == 1) && (!-e $boot_scripts . "/net." . $_[0]->{'name'})) {
 	&unlock_file($boot_scripts . "/net." . $_[0]->{'name'});
 } elsif (($_[0]->{'up'} == 0) && (-e $boot_scripts . "/net." . $_[0]->{'name'})) {
 	&lock_file($boot_scripts . "/net." . $_[0]->{'name'});
-	unlink($boot_scripts . "/net." . $_[0]->{'name'});
+	unlink(&translate_filename($boot_scripts . "/net." . $_[0]->{'name'}));
 	&unlock_file($boot_scripts . "/net." . $_[0]->{'name'});
 }
 
@@ -198,7 +199,7 @@ foreach(sort keys %conf) {
 
 ## [Gentoo] Now delete the boot script
 &lock_file($boot_scripts . "/net." . $_[0]->{'name'});
-unlink($boot_scripts . "/net." . $_[0]->{'name'});
+unlink(&translate_filename($boot_scripts . "/net." . $_[0]->{'name'}));
 &unlock_file($boot_scripts . "/net." . $_[0]->{'name'});
 
 }
@@ -217,128 +218,6 @@ sub valid_boot_address
 return &check_ipaddress($_[0]);
 }
 
-# get_dns_config()
-# Returns a hashtable containing keys nameserver, domain, search & order
-sub get_dns_config
-{
-local $dns;
-open(RESOLV, "/etc/resolv.conf");
-while(<RESOLV>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/nameserver\s+(.*)/) {
-		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
-		}
-	elsif (/domain\s+(\S+)/) {
-		$dns->{'domain'} = [ $1 ];
-		}
-	elsif (/search\s+(.*)/) {
-		$dns->{'domain'} = [ split(/\s+/, $1) ];
-		}
-	}
-close(RESOLV);
-open(SWITCH, "/etc/nsswitch.conf");
-while(<SWITCH>) {
-	s/\r|\n//g;
-	if (/^\s*hosts:\s+(.*)/) {
-		$dns->{'order'} = $1;
-		}
-	}
-close(SWITCH);
-$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
-return $dns;
-}
-
-# save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
-sub save_dns_config
-{
-&lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
-local @resolv = <RESOLV>;
-close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
-foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
-	}
-if ($_[0]->{'domain'}) {
-	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
-		}
-	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
-		}
-	}
-foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
-	}
-close(RESOLV);
-&unlock_file("/etc/resolv.conf");
-
-&lock_file("/etc/nsswitch.conf");
-open(SWITCH, "/etc/nsswitch.conf");
-local @switch = <SWITCH>;
-close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
-foreach (@switch) {
-	if (/^\s*hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
-		}
-	else { print SWITCH $_; }
-	}
-close(SWITCH);
-&unlock_file("/etc/nsswitch.conf");
-}
-
-$max_dns_servers = 3;
-
-# order_input(&dns)
-# Returns HTML for selecting the name resolution order
-sub order_input
-{
-if ($_[0]->{'order'} =~ /\[/) {
-	# Using a complex resolve list
-	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
-	}
-else {
-	# Can select by menus
-	local @o = split(/\s+/, $_[0]->{'order'});
-	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
-	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus", "db" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "DB" );
-	for($i=1; $i<@srcs; $i++) {
-		local $ii = $i-1;
-		$rv .= "<select name=order_$ii>\n";
-		for($j=0; $j<@srcs; $j++) {
-			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-					$srcs[$j],
-					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j] ? $srcn[$j] : "&nbsp;";
-			}
-		$rv .= "</select>\n";
-		}
-	return $rv;
-	}
-}
-
-# parse_order(&dns)
-# Parses the form created by order_input()
-sub parse_order
-{
-if (defined($in{'order'})) {
-	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
-	$_[0]->{'order'} = $in{'order'};
-	}
-else {
-	local($i, @order);
-	for($i=0; defined($in{"order_$i"}); $i++) {
-		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-		}
-	$_[0]->{'order'} = join(" ", @order);
-	}
-}
-
 # get_hostname()
 sub get_hostname
 {
@@ -350,34 +229,25 @@ sub save_hostname
 {
 local %conf;
 &system_logged("hostname $_[0] >/dev/null 2>&1");
-&lock_file("/etc/hostname");
-open(HOST, ">/etc/hostname");
-print HOST $_[0],"\n";
-close(HOST);
-&unlock_file("/etc/hostname");
+&open_lock_tempfile(HOST, ">/etc/hostname");
+&print_tempfile(HOST, $_[0],"\n");
+&close_tempfile(HOST);
 }
 
 # get_domainname()
 sub get_domainname
 {
-local $d = `domainname`;
+local $d;
+&execute_command("domainname", undef, \$d, undef);
 chop($d);
-return $d eq "(none)" ? "" : $d;
+return $d;
 }
 
 # save_domainname(domain)
 sub save_domainname
 {
 local %conf;
-system("domainname \"$_[0]\" >/dev/null 2>&1");
-&read_env_file($network_config, \%conf);
-if ($_[0]) {
-	$conf{'NISDOMAIN'} = $_[0];
-	}
-else {
-	delete($conf{'NISDOMAIN'});
-	}
-&write_env_file($network_config, \%conf);
+&execute_command("domainname ".quotemeta($_[0]));
 }
 
 sub routing_config_files
@@ -388,7 +258,7 @@ return ( $routes_config, $net_config, $sysctl_config );
 sub routing_input
 {
 local (@routes, $i);
-open(ROUTES, $routes_config);
+&open_readfile(ROUTES, $routes_config);
 while(<ROUTES>) {
 	s/#.*$//;
 	s/\r|\n//g;
@@ -489,7 +359,7 @@ for($i=0; defined($in{"dev_$i"}); $i++) {
 	}
 
 # Save routes and routing option
-open(ROUTES, ">$routes_config");
+&open_tempfile(ROUTES, ">$routes_config");
 foreach $r (@routes) {
 	
 	## [Gentoo] Get the default gw's entry and write it to net_config file instead of routes_config
@@ -524,10 +394,10 @@ foreach $r (@routes) {
 
 	## The rest is as it was.
 	} else {
-		print ROUTES join(" ", map { $_ eq '' ? "-" : $_ } @$r),"\n";
+		&print_tempfile(ROUTES, join(" ", map { $_ eq '' ? "-" : $_ } @$r),"\n");
 	}
 }
-close(ROUTES);
+&close_tempfile(ROUTES);
 local $lref = &read_file_lines($sysctl_config);
 for($i=0; $i<@$lref; $i++) {
 	if ($lref->[$i] =~ /^\s*net\.ipv4\.ip_forward\s*=/) {
@@ -556,7 +426,7 @@ sub get_gw_if
 	&backquote_logged("$gwdel 2>&1");
 	local $gwadd = 'route add default gw ' . $ip;
 	&backquote_logged("$gwadd 2>&1") ;
-	open(ROUTE, "route -n |");
+	&open_execute_command(ROUTE, "route -n", 1);
 	while(<ROUTE>) {
 		s/\r|\n//g;
 		if($_ =~ /\d+/) {
