@@ -11,7 +11,7 @@ $hostconfig_file = "/etc/hostconfig";
 sub active_interfaces
 {
 local(@rv, @lines, $l);
-open(IFC, "ifconfig -a |");
+&open_execute_command(IFC, "ifconfig -a", 1);
 while(<IFC>) {
 	s/\r|\n//g;
 	if (/^\S+:/) { push(@lines, $_); }
@@ -95,7 +95,10 @@ sub interface_sync
 # Remove all IP addresses except for the primary one (unless it is being edited)
 local $pri = $_[0]->{$_[1]};
 local $ifconfig = &has_command("ifconfig");
-while(`ifconfig $_[1]` =~ /([\000-\377]*)\s+inet\s+(\d+\.\d+\.\d+\.\d+)/) {
+while(1) {
+	local $out;
+	&execute_command("$ifconfig $_[1]", undef, \$out);
+	last if ($out !~ /([\000-\377]*)\s+inet\s+(\d+\.\d+\.\d+\.\d+)/);
 	last if ($2 eq $pri->{'address'} && $_[2] ne $pri->{'fullname'});
 	&system_logged("$ifconfig $_[1] delete $2 >/dev/null 2>&1");
 	}
@@ -135,7 +138,7 @@ sub boot_interfaces
 local @rv;
 local %virtual_count;
 local $lnum = 0;
-open(IFTAB, $iftab_file);
+&open_readfile(IFTAB, $iftab_file);
 while(<IFTAB>) {
 	s/\r|\n//g;
 	s/#.*$//;
@@ -302,7 +305,7 @@ return &check_ipaddress($_[0]);
 sub get_dns_config
 {
 local $dns;
-open(RESOLV, "/etc/resolv.conf");
+&open_readfile(RESOLV, "/etc/resolv.conf");
 while(<RESOLV>) {
 	s/\r|\n//g;
 	s/#.*$//g;
@@ -326,25 +329,25 @@ return $dns;
 sub save_dns_config
 {
 &lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
+&open_readfile(RESOLV, "/etc/resolv.conf");
 local @resolv = <RESOLV>;
 close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
+&open_tempfile(RESOLV, ">/etc/resolv.conf");
 foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
+	&print_tempfile(RESOLV, "nameserver $_\n");
 	}
 if ($_[0]->{'domain'}) {
 	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
+		&print_tempfile(RESOLV, "search ",join(" ", @{$_[0]->{'domain'}}),"\n");
 		}
 	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
+		&print_tempfile(RESOLV, "domain $_[0]->{'domain'}->[0]\n");
 		}
 	}
 foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
+	&print_tempfile(RESOLV, $_) if (!/^\s*(nameserver|domain|search)\s+/);
 	}
-close(RESOLV);
+&close_tempfile(RESOLV);
 &unlock_file("/etc/resolv.conf");
 }
 
@@ -449,7 +452,7 @@ if (!$found) {
 sub read_hostconfig
 {
 local %rv;
-open(HOST, $hostconfig_file);
+&open_readfile(HOST, $hostconfig_file);
 while(<HOST>) {
 	s/\r|\n//g;
 	s/#.*$//;
