@@ -3,7 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
-require '../ui-lib.pl';
+do '../ui-lib.pl';
 %access = &get_module_acl();
 
 if (-r "$module_root_directory/$gconfig{'os_type'}-$gconfig{'os_version'}-lib.pl") {
@@ -19,7 +19,7 @@ sub list_hosts
 {
 local @rv;
 local $lnum = 0;
-open(HOSTS, $config{'hosts_file'});
+&open_readfile(HOSTS, $config{'hosts_file'});
 while(<HOSTS>) {
 	s/\r|\n//g;
 	s/#.*$//g;
@@ -40,9 +40,9 @@ return @rv;
 # Add a new host to /etc/hosts
 sub create_host
 {
-open(HOSTS, ">>$config{'hosts_file'}");
-print HOSTS $_[0]->{'address'},"\t",join(" ",@{$_[0]->{'hosts'}}),"\n";
-close(HOSTS);
+&open_tempfile(HOSTS, ">>$config{'hosts_file'}");
+&print_tempfile(HOSTS, $_[0]->{'address'},"\t",join(" ",@{$_[0]->{'hosts'}}),"\n");
+&close_tempfile(HOSTS);
 }
 
 # modify_host(&host)
@@ -108,6 +108,11 @@ return int(log($_[0])/log(2));
 # some active interface
 sub module_for_interface
 {
+if (&foreign_check("zones") && $_[0]->{'zone'}) {
+	# Zones virtual interface
+	return { 'module' => 'zones',
+		 'desc' => &text('mod_zones', $_[0]->{'zone'}) };
+	}
 return undef if ($_[0]->{'name'} !~ /^ppp/);	# only for PPP
 if (&foreign_check("ppp-client")) {
 	# Dialup PPP connection
@@ -183,5 +188,69 @@ sub can_create_iface
 return $access{'ifcs'} == 2;
 }
 
+# interface_choice(name, value, blankmode-text, [disabled?])
+# Returns HTML for an interface chooser menu
+sub interface_choice
+{
+local @ifaces = map { $_->{'fullname'} } (&net::active_interfaces(), &net::boot_interfaces());
+@ifaces = sort { $a cmp $b } &unique(@ifaces);
+local $rv = "<select name=$_[0] ".($_[3] ? " disabled=true" : "").">\n";
+local ($i, $found);
+if ($_[2]) {
+	$rv .= sprintf "<option value='' %s> %s\n",
+			$_[1] eq "" ? "checked" : "", $_[2];
+	}
+$found++ if ($_[1] eq "");
+foreach $i (@ifaces) {
+	$rv .= sprintf "<option value=%s %s>%s\n",
+		$i, $_[1] eq $i ? "selected" : "", $i;
+	$found++ if ($_[1] eq $i);
+	}
+#$rv .= "<option value=$_[1] selected>$_[1]\n" if (!$found && $_[1]);
+$rv .= sprintf "<option value=other %s> %s\n",
+		!$found && $_[1] ? "selected" : "", $text{'chooser_other'};
+$rv .= "</select>";
+$rv .= sprintf "<input name=$_[0]_other size=6 value='%s' %s>\n",
+		!$found ? $_[1] : "", $_[3] ? " disabled=true" : "";
+return $rv;
+}
+
+# compute_broadcast(ip, netmask)
+# Returns a computed broadcast address (ip ^ ~netmask)
+sub compute_broadcast
+{
+local $ipnum = &ip_to_integer($_[0]);
+local $nmnum = &ip_to_integer($_[1]);
+return &integer_to_ip($ipnum | (~$nmnum));
+}
+
+# compute_network(ip, netmask)
+# Returns a computed network address (ip & netmask)
+sub compute_network
+{
+local $ipnum = &ip_to_integer($_[0]);
+local $nmnum = &ip_to_integer($_[1]);
+return &integer_to_ip($ipnum & $nmnum);
+}
+
+# ip_to_integer(ip)
+# Given an IP address, returns a 32-bit number
+sub ip_to_integer
+{
+local @ip = split(/\./, $_[0]);
+return ($ip[0]<<24) + ($ip[1]<<16) + ($ip[2]<<8) + ($ip[3]<<0);
+}
+
+# ip_to_integer(integer)
+# Given a 32-bit number, converts it to an IP
+sub integer_to_ip
+{
+return sprintf "%d.%d.%d.%d",
+		($_[0]>>24)&0xff,
+		($_[0]>>16)&0xff,
+		($_[0]>>8)&0xff,
+		($_[0]>>0)&0xff;
+}
+
 1;
 
