@@ -13,7 +13,7 @@ do 'linux-lib.pl';
 sub boot_interfaces
 {
 local(@rv, $f);
-opendir(CONF, $net_scripts_dir);
+opendir(CONF, &translate_filename($net_scripts_dir));
 while($f = readdir(CONF)) {
 	next if ($f !~ /^ifcfg-(\S+)/ || $f eq 'ifcfg-routes' ||
 		 $f =~ /\.sample$/);
@@ -78,7 +78,7 @@ sub delete_interface
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
 &lock_file("$net_scripts_dir/ifcfg-$name");
-unlink("$net_scripts_dir/ifcfg-$name");
+unlink(&translate_filename("$net_scripts_dir/ifcfg-$name"));
 &unlock_file("$net_scripts_dir/ifcfg-$name");
 }
 
@@ -96,128 +96,6 @@ sub valid_boot_address
 return &check_ipaddress($_[0]);
 }
 
-# get_dns_config()
-# Returns a hashtable containing keys nameserver, domain, search & order
-sub get_dns_config
-{
-local $dns;
-open(RESOLV, "/etc/resolv.conf");
-while(<RESOLV>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/^nameserver\s+(.*)/) {
-		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
-		}
-	elsif (/^domain\s+(\S+)/) {
-		$dns->{'domain'} = [ $1 ];
-		}
-	elsif (/^search\s+(.*)/) {
-		$dns->{'domain'} = [ split(/\s+/, $1) ];
-		}
-	}
-close(RESOLV);
-open(SWITCH, "/etc/nsswitch.conf");
-while(<SWITCH>) {
-	s/\r|\n//g;
-	if (/hosts:\s+(.*)/) {
-		$dns->{'order'} = $1;
-		}
-	}
-close(SWITCH);
-$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
-return $dns;
-}
-
-# save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
-sub save_dns_config
-{
-&lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
-local @resolv = <RESOLV>;
-close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
-foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
-	}
-if ($_[0]->{'domain'}) {
-	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
-		}
-	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
-		}
-	}
-foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
-	}
-close(RESOLV);
-&unlock_file("/etc/resolv.conf");
-
-&lock_file("/etc/nsswitch.conf");
-open(SWITCH, "/etc/nsswitch.conf");
-local @switch = <SWITCH>;
-close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
-foreach (@switch) {
-	if (/hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
-		}
-	else { print SWITCH $_; }
-	}
-close(SWITCH);
-&unlock_file("/etc/nsswitch.conf");
-}
-
-$max_dns_servers = 3;
-
-# order_input(&dns)
-# Returns HTML for selecting the name resolution order
-sub order_input
-{
-if ($_[0]->{'order'} =~ /\[/) {
-	# Using a complex resolve list
-	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
-	}
-else {
-	# Can select by menus
-	local @o = split(/\s+/, $_[0]->{'order'});
-	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
-	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+" );
-	for($i=1; $i<@srcs; $i++) {
-		local $ii = $i-1;
-		$rv .= "<select name=order_$ii>\n";
-		for($j=0; $j<@srcs; $j++) {
-			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-					$srcs[$j],
-					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j] ? $srcn[$j] : "&nbsp;";
-			}
-		$rv .= "</select>\n";
-		}
-	return $rv;
-	}
-}
-
-# parse_order(&dns)
-# Parses the form created by order_input()
-sub parse_order
-{
-if (defined($in{'order'})) {
-	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
-	$_[0]->{'order'} = $in{'order'};
-	}
-else {
-	local($i, @order);
-	for($i=0; defined($in{"order_$i"}); $i++) {
-		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-		}
-	$_[0]->{'order'} = join(" ", @order);
-	}
-}
-
 sub get_hostname
 {
 return &get_system_hostname(1);
@@ -228,11 +106,9 @@ sub save_hostname
 {
 local %conf;
 &system_logged("hostname $_[0] >/dev/null 2>&1");
-&lock_file("/etc/HOSTNAME");
-open(HOST, ">/etc/HOSTNAME");
-print HOST $_[0],"\n";
-close(HOST);
-&unlock_file("/etc/HOSTNAME");
+&open_lock_tempfile(HOST, ">/etc/HOSTNAME");
+&print_tempfile(HOST, $_[0],"\n");
+&close_tempfile(HOST);
 &lock_file($network_config);
 &read_file($network_config, \%conf);
 $conf{'HOSTNAME'} = $_[0];
@@ -243,25 +119,25 @@ $conf{'HOSTNAME'} = $_[0];
 # get_domainname()
 sub get_domainname
 {
-local $d = `domainname`;
+local $d;
+&execute_command("domainname", undef, \$d, undef);
 chop($d);
-return $d eq "(none)" ? "" : $d;
+return $d;
 }
 
 # save_domainname(domain)
 sub save_domainname
 {
 local %conf;
-system("domainname \"$_[0]\" >/dev/null 2>&1");
-&read_file("$network_config", \%conf);
+&execute_command("domainname ".quotemeta($_[0]));
+&read_env_file($network_config, \%conf);
 if ($_[0]) {
 	$conf{'NISDOMAIN'} = $_[0];
 	}
 else {
 	delete($conf{'NISDOMAIN'});
 	}
-&write_file("$network_config", \%conf);
-# XXX need to update nis.conf
+&write_env_file($network_config, \%conf);
 }
 
 sub routing_config_files
@@ -294,7 +170,7 @@ printf "<input type=radio name=forward value=0 %s> $text{'no'}</td> </tr>\n",
 
 print "<tr> <td valign=top><b>$text{'routes_script'}</b></td> <td>\n";
 print "<textarea name=script rows=4 cols=60>\n";
-open(SCRIPT, $static_route_config);
+&open_readfile(SCRIPT, $static_route_config);
 while(<SCRIPT>) { print; }
 close(SCRIPT);
 print "</textarea></td> </tr>\n";
@@ -321,12 +197,10 @@ if (!$in{'gateway_def'}) {
 &write_file($network_config, \%conf);
 &unlock_file($network_config);
 
-&lock_file($static_route_config);
-open(SCRIPT, ">$static_route_config");
+&open_lock_tempfile(SCRIPT, ">$static_route_config");
 $in{'script'} =~ s/\r//g;
-print SCRIPT $in{'script'};
-close(SCRIPT);
-&unlock_file($static_route_config);
+&print_tempfile(SCRIPT, $in{'script'});
+&close_tempfile(SCRIPT);
 &system_logged("chmod +x $static_route_config");
 }
 
@@ -344,7 +218,7 @@ return ( (map { "$net_scripts_dir/$_" } grep { /^ifcfg-/ } @f),
 # Apply the interface and routing settings
 sub apply_network
 {
-system("(cd / ; /etc/rc.d/init.d/network stop ; /etc/rc.d/init.d/network start) >/dev/null 2>&1");
+&system_logged("(cd / ; /etc/rc.d/init.d/network stop ; /etc/rc.d/init.d/network start) >/dev/null 2>&1");
 }
 
 # apply_interface(&iface)
@@ -361,7 +235,7 @@ return $? ? $out : undef;
 sub get_default_gateway
 {
 &read_file($network_config, \%conf);
-opendir(CONF, $net_scripts_dir);
+opendir(CONF, &translate_filename($net_scripts_dir));
 local $f;
 while($f = readdir(CONF)) {
 	next if ($f !~ /^ifcfg-(\S+)/);
@@ -380,7 +254,7 @@ return ( );
 # in the boot time settings.
 sub set_default_gateway
 {
-opendir(CONF, $net_scripts_dir);
+opendir(CONF, &translate_filename($net_scripts_dir));
 local $f;
 while($f = readdir(CONF)) {
 	next if ($f !~ /^ifcfg-(\S+)/);
