@@ -13,7 +13,7 @@ sub boot_interfaces
 {
 local(@rv, $f);
 local @active = &active_interfaces();
-opendir(CONF, $net_scripts_dir);
+opendir(CONF, &translate_filename($net_scripts_dir));
 while($f = readdir(CONF)) {
 	if ($f =~ /^ifcfg-eth-id-([a-f0-9:]+)$/i) {
 		# An interface identified by MAC address!
@@ -132,7 +132,7 @@ local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
 local $file = $_[0]->{'file'} || "$net_scripts_dir/ifcfg-$name";
 &lock_file($file);
-unlink($file);
+unlink(&translate_filename($file));
 &unlock_file($file);
 }
 
@@ -150,128 +150,6 @@ sub valid_boot_address
 return &check_ipaddress($_[0]);
 }
 
-# get_dns_config()
-# Returns a hashtable containing keys nameserver, domain, search & order
-sub get_dns_config
-{
-local $dns;
-open(RESOLV, "/etc/resolv.conf");
-while(<RESOLV>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/nameserver\s+(.*)/) {
-		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
-		}
-	elsif (/domain\s+(\S+)/) {
-		$dns->{'domain'} = [ $1 ];
-		}
-	elsif (/search\s+(.*)/) {
-		$dns->{'domain'} = [ split(/\s+/, $1) ];
-		}
-	}
-close(RESOLV);
-open(SWITCH, "/etc/nsswitch.conf");
-while(<SWITCH>) {
-	s/\r|\n//g;
-	if (/^\s*hosts:\s+(.*)/) {
-		$dns->{'order'} = $1;
-		}
-	}
-close(SWITCH);
-$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
-return $dns;
-}
-
-# save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
-sub save_dns_config
-{
-&lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
-local @resolv = <RESOLV>;
-close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
-foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
-	}
-if ($_[0]->{'domain'}) {
-	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
-		}
-	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
-		}
-	}
-foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
-	}
-close(RESOLV);
-&unlock_file("/etc/resolv.conf");
-
-&lock_file("/etc/nsswitch.conf");
-open(SWITCH, "/etc/nsswitch.conf");
-local @switch = <SWITCH>;
-close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
-foreach (@switch) {
-	if (/^\s*hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
-		}
-	else { print SWITCH $_; }
-	}
-close(SWITCH);
-&unlock_file("/etc/nsswitch.conf");
-}
-
-$max_dns_servers = 3;
-
-# order_input(&dns)
-# Returns HTML for selecting the name resolution order
-sub order_input
-{
-if ($_[0]->{'order'} =~ /\[/) {
-	# Using a complex resolve list
-	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
-	}
-else {
-	# Can select by menus
-	local @o = split(/\s+/, $_[0]->{'order'});
-	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
-	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus", "db" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "DB" );
-	for($i=1; $i<@srcs; $i++) {
-		local $ii = $i-1;
-		$rv .= "<select name=order_$ii>\n";
-		for($j=0; $j<@srcs; $j++) {
-			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-					$srcs[$j],
-					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j] ? $srcn[$j] : "&nbsp;";
-			}
-		$rv .= "</select>\n";
-		}
-	return $rv;
-	}
-}
-
-# parse_order(&dns)
-# Parses the form created by order_input()
-sub parse_order
-{
-if (defined($in{'order'})) {
-	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
-	$_[0]->{'order'} = $in{'order'};
-	}
-else {
-	local($i, @order);
-	for($i=0; defined($in{"order_$i"}); $i++) {
-		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-		}
-	$_[0]->{'order'} = join(" ", @order);
-	}
-}
-
 # get_hostname()
 sub get_hostname
 {
@@ -283,11 +161,9 @@ sub save_hostname
 {
 local %conf;
 &system_logged("hostname $_[0] >/dev/null 2>&1");
-&lock_file("/etc/HOSTNAME");
-open(HOST, ">/etc/HOSTNAME");
-print HOST $_[0],"\n";
-close(HOST);
-&unlock_file("/etc/HOSTNAME");
+&open_lock_tempfile(HOST, ">/etc/HOSTNAME");
+&print_tempfile(HOST, $_[0],"\n");
+&close_tempfile(HOST);
 }
 
 sub routing_config_files
@@ -300,7 +176,7 @@ return ( $routes_config, $sysctl_config );
 sub get_routes_config
 {
 local (@routes);
-open(ROUTES, $routes_config);
+&open_readfile(ROUTES, $routes_config);
 while(<ROUTES>) {
 	s/#.*$//;
 	s/\r|\n//g;
@@ -314,15 +190,15 @@ return @routes;
 # save_routes_config(&routes)
 sub save_routes_config
 {
-open(ROUTES, ">$routes_config");
+&open_tempfile(ROUTES, ">$routes_config");
 foreach $r (@{$_[0]}) {
-	print ROUTES join(" ",
+	&print_tempfile(ROUTES, join(" ",
 		$r->[0] || "-",
 		$r->[1] || "-",
 		$r->[2] || "-",
-		$r->[3] || "-"),"\n";
+		$r->[3] || "-"),"\n");
 	}
-close(ROUTES);
+&close_tempfile(ROUTES);
 }
 
 sub routing_input
@@ -468,7 +344,7 @@ return ( (map { "$net_scripts_dir/$_" } grep { /^ifcfg-/ } @f),
 # Apply the interface and routing settings
 sub apply_network
 {
-system("(cd / ; /etc/init.d/network stop ; /etc/init.d/network start) >/dev/null 2>&1");
+&system_logged("(cd / ; /etc/init.d/network stop ; /etc/init.d/network start) >/dev/null 2>&1");
 }
 
 1;
