@@ -6,7 +6,7 @@
 sub active_interfaces
 {
 local(@rv, @lines, $l);
-open(IFC, "ifconfig -a |");
+&open_execute_command(IFC, "ifconfig -a", 1);
 while(<IFC>) {
 	s/\r|\n//g;
 	if (/^\S+:/) { push(@lines, $_); }
@@ -41,7 +41,7 @@ sub activate_interface
 {
 local $a = $_[0];
 if ($a->{'virtual'} eq "") {
-	local $out = `ifconfig $a->{'name'} plumb 2>&1`;
+	local $out = &backquote_logged("ifconfig $a->{'name'} plumb 2>&1");
 	if ($out) { &error("Interface '$a->{'name'}' does not exist"); }
 	}
 local $cmd = "ifconfig $a->{'name'}";
@@ -149,13 +149,11 @@ sub save_interface
 {
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
-&lock_file("/etc/hostname.$name");
-open(IFACE, ">/etc/hostname.$name");
+&open_lock_tempfile(IFACE, ">/etc/hostname.$name");
 if (!$_[0]->{'dhcp'}) {
-	print IFACE $_[0]->{'address'},"\n";
+	&print_tempfile(IFACE, $_[0]->{'address'},"\n");
 	}
-close(IFACE);
-&unlock_file("/etc/hostname.$name");
+&close_tempfile(IFACE);
 }
 
 # delete_interface(&details)
@@ -165,7 +163,7 @@ sub delete_interface
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
 &lock_file("/etc/hostname.$name");
-unlink("/etc/hostname.$name");
+unlink(&translate_filename("/etc/hostname.$name"));
 &unlock_file("/etc/hostname.$name");
 }
 
@@ -242,36 +240,38 @@ sub save_dns_config
 open(RESOLV, "/etc/resolv.conf");
 local @resolv = <RESOLV>;
 close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
+&open_tempfile(RESOLV, ">/etc/resolv.conf");
 foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
+	&print_tempfile(RESOLV, "nameserver $_\n");
 	}
 if ($_[0]->{'domain'}) {
 	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
+		&print_tempfile(RESOLV, "search ",join(" ", @{$_[0]->{'domain'}}),"\n");
 		}
 	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
+		&print_tempfile(RESOLV, "domain $_[0]->{'domain'}->[0]\n");
 		}
 	}
 foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
+	&print_tempfile(RESOLV, $_) if (!/^\s*(nameserver|domain|search)\s+/);
 	}
-close(RESOLV);
+&close_tempfile(RESOLV);
 &unlock_file("/etc/resolv.conf");
 
 &lock_file("/etc/nsswitch.conf");
 open(SWITCH, "/etc/nsswitch.conf");
 local @switch = <SWITCH>;
 close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
+&open_tempfile(SWITCH, ">/etc/nsswitch.conf");
 foreach (@switch) {
 	if (/hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
+		&print_tempfile(SWITCH, "hosts:\t$_[0]->{'order'}\n");
+		}
+	else {
+		&print_tempfile(SWITCH, $_);
 		}
-	else { print SWITCH $_; }
 	}
-close(SWITCH);
+&close_tempfile(SWITCH);
 &unlock_file("/etc/nsswitch.conf");
 }
 
@@ -337,7 +337,8 @@ sub save_hostname
 # get_domainname()
 sub get_domainname
 {
-local $d = `domainname`;
+local $d;
+&execute_command("domainname", undef, \$d, undef);
 chop($d);
 return $d;
 }
@@ -345,13 +346,15 @@ return $d;
 # save_domainname(domain)
 sub save_domainname
 {
-system("domainname \"$_[0]\" >/dev/null 2>&1");
+&execute_command("domainname ".quotemeta($_[0]));
 if ($_[0]) {
-	open(DOMAIN, ">/etc/defaultdomain");
-	print DOMAIN $_[0],"\n";
-	close(DOMAIN);
+	&open_tempfile(DOMAIN, ">/etc/defaultdomain");
+	&print_tempfile(DOMAIN, $_[0],"\n");
+	&close_tempfile(DOMAIN);
+	}
+else {
+	unlink(&translate_filename("/etc/defaultdomain"));
 	}
-else { unlink("/etc/defaultdomain"); }
 }
 
 sub routing_config_files
@@ -363,7 +366,7 @@ sub routing_input
 {
 # show default router(s) input
 local(@defrt);
-open(DEFRT, "/etc/defaultrouter");
+&open_readfile(DEFRT, "/etc/defaultrouter");
 while(<DEFRT>) {
 	s/#.*$//g;
 	if (/(\S+)/) { push(@defrt, $1); }
@@ -394,26 +397,26 @@ foreach $d (@defrt) {
 	}
 &lock_file("/etc/defaultrouter");
 if (@defrt) {
-	open(DEFRT, ">/etc/defaultrouter");
-	foreach $d (@defrt) { print DEFRT $d,"\n"; }
-	close(DEFRT);
+	&open_tempfile(DEFRT, ">/etc/defaultrouter");
+	foreach $d (@defrt) { &print_tempfile(DEFRT, $d,"\n"); }
+	&close_tempfile(DEFRT);
 	}
-else { unlink("/etc/defaultrouter"); }
+else { unlink(&translate_filename("/etc/defaultrouter")); }
 &unlock_file("/etc/defaultrouter");
 
 &lock_file("/etc/gateways");
 &lock_file("/etc/notrouter");
 if ($in{'router'} == 0) {
 	&create_empty_file("/etc/gateways");
-	unlink("/etc/notrouter");
+	unlink(&translate_filename("/etc/notrouter"));
 	}
 elsif ($in{'router'} == 2) {
 	&create_empty_file("/etc/notrouter");
-	unlink("/etc/gateways");
+	unlink(&translate_filename("/etc/gateways"));
 	}
 else {
-	unlink("/etc/gateways");
-	unlink("/etc/notrouter");
+	unlink(&translate_filename("/etc/gateways"));
+	unlink(&translate_filename("/etc/notrouter"));
 	}
 &unlock_file("/etc/gateways");
 &unlock_file("/etc/notrouter");
@@ -423,8 +426,8 @@ else {
 sub create_empty_file
 {
 if (!-r $_[0]) {
-	open(EMPTY,">$_[0]");
-	close(EMPTY);
+	&open_tempfile(EMPTY,">$_[0]");
+	&close_tempfile(EMPTY);
 	}
 }
 
