@@ -12,9 +12,10 @@ $| = 1;
 open(OS, $oslist) || die "failed to open $oslist : $!";
 while(<OS>) {
 	chop;
-	if (/^([^\t]+)\t+([^\t]+)\t+(\S+)\t+(\S+)\t*(.*)$/) {
+	if (/^([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+([^\t]+)\t*(.*)$/) {
 		push(@list, [ $1, $2, $3, $4, $5 ]);
 		push(@names, $1) if (!$donename{$1}++);
+		$names_to_real{$1} ||= $3;
 		}
 	}
 close(OS);
@@ -29,12 +30,22 @@ if ($auto) {
 		}
 	$uname = `uname -a`;
 	foreach $o (@list) {
-		if ($o->[4] && eval $o->[4]) {
-			# Got a match!
+		if ($o->[4] && eval "$o->[4]") {
+			# Got a match! Resolve the versions
 			$ver = $o;
+			if ($ver->[1] =~ /\$/) {
+				$ver->[1] = eval "($o->[4]); $ver->[1]";
+				}
+			if ($ver->[3] =~ /\$/) {
+				$ver->[3] = eval "($o->[4]); $ver->[3]";
+				}
 			last;
 			}
+		if ($@) {
+			print STDERR "Error parsing $o->[4]\n";
+			}
 		}
+
 	if (!$ver) {
 		if ($auto == 1) {
 			# Failed .. give up
@@ -124,31 +135,19 @@ EOF
 
 		# Ask for the operating system version
 		$name = $names[$osnum-1];
-		@vers = grep { $_->[0] eq $name } @list; 
 		print <<EOF;
-Please choose which version of $name you are running, by entering
-the number next to it from the list below
-$dashes
+Please enter the version of $name you are running
 EOF
-		for($i=0; $i<@vers; $i++) {
-			printf " %2d) %-34.34s ", $i+1, $name." ".$vers[$i]->[1];
-			print "\n" if ($i%2);
-			}
-		print "\n" if ($i%2);
-		print $dashes,"\n";
 		print "Version: ";
 		chop($vnum = <STDIN>);
-		if ($vnum !~ /^\d+$/) {
-			print "ERROR: You must enter the number next to your operating\n";
-			print "system version, not its name or version number.\n\n";
-			exit 9;
-			}
-		if ($vnum < 1 || $vnum > @vers) {
-			print "ERROR: $vnum is not a valid operating system number.\n\n";
+		if ($vnum !~ /^\S+$/) {
+			print "ERROR: An operating system number cannot contain\n\n";
+			print "spaces. It must be like 2.1 or ES4.0.\n";
 			exit 10;
 			}
 		print "\n";
-		$ver = $vers[$vnum-1];
+		$ver = [ $names_to_real{$name}, $vnum,
+			 $name, $vnum ];
 		}
 	}
 
