@@ -17,10 +17,10 @@ sub parse_module_args
 {
 local $file = "/etc/shells";
 &lock_file($file);
-open(FILE, ">$file");
+&open_tempfile(FILE, ">$file");
 $in{'shells'} =~ s/\r//g;
 $in{'shells'} =~ s/\s*$/\n/;
-print FILE $in{'shells'};
-close(FILE);
+&print_tempfile(FILE, $in{'shells'});
+&close_tempfile(FILE);
 &unlock_file($file);
 }
