@@ -25,6 +25,14 @@ printf "<td><input type=radio name=run value=1 %s> %s\n",
 	$_[0]->{'run'} ? 'checked' : '', $text{'yes'};
 printf "<input type=radio name=run value=0 %s> %s</td> </tr>\n",
 	$_[0]->{'run'} ? '' : 'checked', $text{'no'};
+
+print "<tr> <td><b>$text{'acl_only'}</b></td>\n";
+printf "<td><input type=radio name=only value=1 %s> %s\n",
+	$_[0]->{'only'} ? 'checked' : '', $text{'yes'};
+printf "<input type=radio name=only value=0 %s> %s</td>\n",
+	$_[0]->{'only'} ? '' : 'checked', $text{'no'};
+
+print "</tr>\n";
 }
 
 # acl_security_save(&options)
@@ -34,5 +42,6 @@ sub acl_security_save
 $_[0]->{'uid'} = $in{'uid_def'} ? -1 : getpwnam($in{'uid'});
 $_[0]->{'edit'} = $in{'edit'};
 $_[0]->{'run'} = $in{'run'};
+$_[0]->{'only'} = $in{'only'};
 }
 
