@@ -121,11 +121,10 @@ return split(/\s+/, $out);
 # Returns the filehandles and names for a pty and tty
 sub get_new_pty
 {
-if (-r "/dev/ptmx" && -d "/dev/pts" && !$<) {
+if (-r "/dev/ptmx" && -d "/dev/pts" && open(PTMX, "+>/dev/ptmx")) {
 	# Can use new-style PTY number allocation device
 	local $unl;
 	local $ptn;
-	open(PTMX, "+>/dev/ptmx") || &error("Failed to open /dev/ptmx : $!");
 
 	# ioctl to unlock the PTY (TIOCSPTLCK)
 	$unl = pack("i", 0);
@@ -156,6 +155,7 @@ else {
 		$devstyle = 0;
 		}
 	local ($pty, $tty);
+	print STDERR "considering ",scalar(@ptys)," ptys\n";
 	foreach $pty (@ptys) {
 		open(PTY, "+>$pty") || next;
 		local $tty = $pty;
@@ -174,11 +174,13 @@ else {
 			}
 		else {
 			# Must open now ..
+			print STDERR "opening tty $pty $tty\n";
 			open(TTY, "+>$tty");
 			select(TTY); $| = 1; select($old);
 			return (*PTY, *TTY, $pty, $tty);
 			}
 		}
+	print STDERR "no tty found!!\n";
 	return ();
 	}
 }
