@@ -20,6 +20,7 @@ else {
 	%access = &get_module_acl();
 	map { $hide{$_}++ } split(/\s+/, $access{'hide'});
 	$index_file = "$module_config_directory/index";
+	$user_processes_only = $access{'only'};
 	}
 
 sub process_info
@@ -33,7 +34,8 @@ sub index_links
 {
 local(%linkname, $l);
 print "<b>$text{'index_display'} : </b>&nbsp;&nbsp;\n";
-foreach $l ("tree", "user", "size", "cpu", "search", "run") {
+foreach $l ("tree", "user", "size", "cpu", ($has_zone ? ("zone") : ()),
+	    "search", "run") {
 	next if ($l eq "run" && !$access{'run'});
 	if ($l ne $_[0]) { print "<a href=index_$l.cgi>"; }
 	else { print "<b>"; }
@@ -66,6 +68,7 @@ sub switch_acl_uid
 return if ($module_info{'usermin'});	# already switched!
 if ($access{'uid'} < 0) {
 	local @u = getpwnam($remote_user);
+	@u || &error("Failed to find user $remote_user");
 	$( = $u[3]; $) = "$u[3] $u[3]";
 	($>, $<) = ($u[2], $u[2]);
 	}
@@ -198,47 +201,85 @@ do {    local $oldexit = $?;
 # Starts the given command in a new pty and returns the pty filehandle and PID
 sub pty_process_exec
 {
-local ($ptyfh, $ttyfh, $pty, $tty) = &get_new_pty();
-local $pid = fork();
-if (!$pid) {
-	if (defined(&close_controlling_pty)) {
-		&close_controlling_pty();
+eval "use IO::Pty";
+if (!$@) {
+	# Use the IO::Pty perl module if installed
+	local $ptyfh = new IO::Pty;
+	if (!$ptyfh) {
+		&error("Failed to create new PTY with IO::Pty");
 		}
-
-	setsid();	# create a new session group
-
-	if (!$ttyfh) {
-		# Needs to be opened, as get_new_pty on linux cannot do
-		# this so soon
-		$ttyfh = "TTY";
-		if ($_[1]) {
-			chown($_[1], $_[2], $tty);
+	local $ttyfh = $ptyfh->slave();
+	local $tty = $ptyfh->ttyname();
+	local $pid = fork();
+	if (!$pid) {
+		if (defined(&close_controlling_pty)) {
+			&close_controlling_pty();
 			}
-		open($ttyfh, "+<$tty") || &error("Failed to open $tty : $!");
-		}
+		setsid();	# create a new session group
+		$ptyfh->make_slave_controlling_terminal();
 
-	if (defined(&open_controlling_pty)) {
-		&open_controlling_pty($ptyfh, $ttyfh, $pty, $tty);
-		}
+		close(STDIN); close(STDOUT); close(STDERR);
+		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
+		if ($_[1]) {
+			$( = $u[3]; $) = "$_[2] $_[2]";
+			($>, $<) = ($_[1], $_[1]);
+			}
 
-	close(STDIN); close(STDOUT); close(STDERR);
-	untie(*STDIN); untie(*STDOUT); untie(*STDERR);
-	#setpgrp(0, $$);
-	if ($_[1]) {
-		$( = $u[3]; $) = "$_[2] $_[2]";
-		($>, $<) = ($_[1], $_[1]);
+		open(STDIN, "<$tty");
+		open(STDOUT, ">$tty");
+		open(STDERR, ">&STDOUT");
+		close($ptyfh);
+		exec($_[0]);
+		print "Exec failed : $!\n";
+		exit 1;
 		}
+	$ptyfh->close_slave();
+	return ($ptyfh, $pid);
+	}
+else {
+	# Need to create a PTY using built-in Webmin code
+	local ($ptyfh, $ttyfh, $pty, $tty) = &get_new_pty();
+	local $pid = fork();
+	if (!$pid) {
+		if (defined(&close_controlling_pty)) {
+			&close_controlling_pty();
+			}
 
-	open(STDIN, "<$tty");
-	open(STDOUT, ">&$ttyfh");
-	open(STDERR, ">&STDOUT");
-	close($ptyfh);
-	exec($_[0]);
-	print "Exec failed : $!\n";
-	exit 1;
+		setsid();	# create a new session group
+
+		if (!$ttyfh) {
+			# Needs to be opened, as get_new_pty on linux cannot do
+			# this so soon
+			$ttyfh = "TTY";
+			if ($_[1]) {
+				chown($_[1], $_[2], $tty);
+				}
+			open($ttyfh, "+<$tty") || &error("Failed to open $tty : $!");
+			}
+
+		if (defined(&open_controlling_pty)) {
+			&open_controlling_pty($ptyfh, $ttyfh, $pty, $tty);
+			}
+
+		close(STDIN); close(STDOUT); close(STDERR);
+		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
+		#setpgrp(0, $$);
+		if ($_[1]) {
+			$( = $u[3]; $) = "$_[2] $_[2]";
+			($>, $<) = ($_[1], $_[1]);
+			}
+
+		open(STDIN, "<$tty");
+		open(STDOUT, ">&$ttyfh");
+		open(STDERR, ">&STDOUT");
+		close($ptyfh);
+		exec($_[0]);
+		print "Exec failed : $!\n";
+		exit 1;
+		}
+	close($ttyfh);
+	return ($ptyfh, $pid);
 	}
-close($ttyfh);
-return ($ptyfh, $pid);
 }
 
 # pty_process_exec_logged(..)
@@ -275,6 +316,21 @@ close(LSOF);
 return @rv;
 }
 
+# find_ip_processes(ip)
+# Returns all processes using some IP address
+sub find_ip_processes
+{
+local @rv;
+open(LSOF, "lsof -i '\@$_[0]' |");
+while(<LSOF>) {
+	if (/^(\S+)\s+(\d+)/) {
+		push(@rv, $2);
+		}
+	}
+close(LSOF);
+return @rv;
+}
+
 # find_process_sockets(pid)
 # Returns all network connections made by some process
 sub find_process_sockets
@@ -372,6 +428,21 @@ return $out =~ /average(s)?:\s+([0-9\.]+),?\s+([0-9\.]+),?\s+([0-9\.]+)/i ?
 		( $2, $3, $4 ) : ( );
 }
 
+# find_subprocesses(&proc, [&plist])
+# Returns a list of all processes under the one given
+sub find_subprocesses
+{
+local $proc = $_[0];
+local @plist = $_[1] ? @{$_[1]} : &list_processes();
+local @sp = grep { $_->{'ppid'} &&
+		   $_->{'ppid'} == $proc->{'pid'} } @plist;
+local (@rv, $sp);
+foreach $sp (@sp) {
+	push(@rv, $sp, &find_subprocesses($sp, \@plist));
+	}
+return @rv;
+}
+
 
 
 1;
