@@ -14,8 +14,9 @@ else {
 	}
 
 foreach $m (@mods) {
+	$mdir = &module_root_directory($m->{'dir'});
 	if (&check_os_support($m) &&
-	    -r "$root_directory/$m->{'dir'}/postinstall.pl") {
+	    -r "$mdir/postinstall.pl") {
 		# Call this module's postinstall function
 		eval {
 			&foreign_require($m->{'dir'}, "postinstall.pl");
