@@ -67,8 +67,7 @@ foreach $s ($_[0] ? @{$_[0]} : &list_servers()) {
 	foreach $gn (split(/\t+/, $s->{'group'})) {
 		local $grp = $gmap{$gn};
 		if (!$grp) {
-			$gmap{$s->{'group'}} = $grp = { 'name' => $gn,
-							'type' => 0 };
+			$gmap{$gn} = $grp = { 'name' => $gn, 'type' => 0 };
 			push(@rv, $grp);
 			}
 		push(@{$grp->{'members'}}, $s->{'host'});
@@ -142,6 +141,7 @@ else {
 		  [ 'mandrake', 'Mandrake Linux', 'mandrake-linux' ],
 		  [ 'msc', 'MSC.Linux', 'msc-linux' ],
 		  [ 'cobalt', 'Cobalt Linux', 'cobalt-linux' ],
+		  [ 'slackware', 'Slackware Linux', 'slackware-linux' ],
 		  [ 'linux', 'Linux', '.*-linux' ],
 		  [ 'freebsd', 'FreeBSD', 'freebsd' ],
 		  [ 'solaris', 'Solaris', 'solaris' ],
