@@ -7,16 +7,23 @@
 do '../web-lib.pl';
 &init_config();
 do '../ui-lib.pl';
-do "$gconfig{'os_type'}-lib.pl";
+if ($gconfig{'os_type'} =~ /-linux$/) {
+	do "linux-lib.pl";
+	}
+else {
+	do "$gconfig{'os_type'}-lib.pl";
+	}
+do "md5-lib.pl";
 
 @random_password_chars = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9' );
+$disable_string = $config{'lock_prepend'} eq "" ? "!" : $config{'lock_prepend'};
 
 # password_file(file)
 # Returns true if some file looks like a valid Unix password file
 sub password_file
 {
 if (!$_[0]) { return 0; }
-elsif (open(SHTEST, $_[0])) {
+elsif (&open_readfile(SHTEST, $_[0])) {
 	local($line);
 	$line = <SHTEST>;
 	close(SHTEST);
@@ -43,7 +50,7 @@ local $pft = &passfiles_type();
 if ($pft == 1) {
 	# read the master.passwd file only
 	$lnum = 0;
-	open(PASSWD, $config{'master_file'});
+	&open_readfile(PASSWD, $config{'master_file'});
 	while(<PASSWD>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -61,7 +68,7 @@ if ($pft == 1) {
 	}
 elsif ($pft == 6) {
 	# Read netinfo dump
-	open(PASSWD, "nidump passwd '$netinfo_domain' |");
+	&open_readfile(PASSWD, "nidump passwd '$netinfo_domain' |");
 	while(<PASSWD>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -79,7 +86,7 @@ elsif ($pft == 6) {
 else {
 	# start by reading /etc/passwd
 	$lnum = 0;
-	open(PASSWD, $config{'passwd_file'});
+	&open_readfile(PASSWD, $config{'passwd_file'});
 	while(<PASSWD>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -97,7 +104,7 @@ else {
 	if ($pft == 2 || $pft == 5) {
 		# read the shadow file data
 		$lnum = 0;
-		open(SHADOW, $config{'shadow_file'});
+		&open_readfile(SHADOW, $config{'shadow_file'});
 		while(<SHADOW>) {
 			s/\r|\n//g;
 			if (/\S/ && !/^[#\+\-]/) {
@@ -127,7 +134,7 @@ else {
 		# read the AIX security passwd file
 		local $lastuser;
 		local $lnum = 0;
-		open(SECURITY, $config{'shadow_file'});
+		&open_readfile(SECURITY, $config{'shadow_file'});
 		while(<SECURITY>) {
 			s/\s*$//;
 			if (/^\s*(\S+):/) {
@@ -152,7 +159,7 @@ else {
 		close(SECURITY);
 
 		# read the AIX security user file
-		open(USER, $config{'aix_user_file'});
+		&open_readfile(USER, $config{'aix_user_file'});
 		while(<USER>) {
 			s/\s*$//;
 			if (/^\s*(\S+):/) {
@@ -433,7 +440,7 @@ $lnum = 0;
 local $gft = &groupfiles_type();
 if ($gft == 5) {
 	# Get groups from netinfo
-	open(GROUP, "nidump group '$netinfo_domain' |");
+	&open_readfile(GROUP, "nidump group '$netinfo_domain' |");
 	while(<GROUP>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -448,7 +455,7 @@ if ($gft == 5) {
 	}
 else {
 	# Read the standard group file
-	open(GROUP, $config{'group_file'});
+	&open_readfile(GROUP, $config{'group_file'});
 	while(<GROUP>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -465,7 +472,7 @@ else {
 if ($gft == 2) {
 	# read the gshadow file data
 	$lnum = 0;
-	open(SHADOW, $config{'gshadow_file'});
+	&open_readfile(SHADOW, $config{'gshadow_file'});
 	while(<SHADOW>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -489,7 +496,7 @@ elsif ($gft == 4) {
 	# read the AIX group data
 	local $lastgroup;
 	local $lnum = 0;
-	open(SECURITY, $config{'gshadow_file'});
+	&open_readfile(SECURITY, $config{'gshadow_file'});
 	while(<SECURITY>) {
 		s/\s*$//;
 		if (/^\s*(\S+):/) {
@@ -1055,7 +1062,8 @@ if ($config{'passwd_same'}) {
 	}
 if ($config{'passwd_dict'} && $_[0] =~ /^[A-Za-z\'\-]+$/ &&
     (&has_command("ispell") || &has_command("spell"))) {
-	local $temp = &tempname();
+	# Call spell or ispell to check for dictionary words
+	local $temp = &transname();
 	open(TEMP, ">$temp");
 	print TEMP $_[0],"\n";
 	close(TEMP);
@@ -1077,6 +1085,15 @@ if ($config{'passwd_dict'} && $_[0] =~ /^[A-Za-z\'\-]+$/ &&
 	unlink($temp);
 	return &text('usave_epasswd_dict') if (!$unknown);
 	}
+if ($config{'passwd_prog'}) {
+	# Run external program with username and password as args
+	local $qu = quotemeta($_[1]);
+	local $qp = quotemeta($_[0]);
+	local $out = `$config{'passwd_prog'} $qu $qp 2>&1 </dev/null`;
+	if ($?) {
+		return $out;
+		}
+	}
 return undef;
 }
 
@@ -1166,109 +1183,6 @@ foreach $e (keys %ENV) {
 	}
 }
 
-# check_md5()
-# Returns a perl module name if the needed perl module(s) for MD5 encryption
-# are not installed, or undef if they are
-sub check_md5
-{
-eval "use MD5";
-if (!$@) {
-	eval "use Digest::MD5";
-	if ($@) {
-		return "Digest::MD5";
-		}
-	}
-return undef;
-}
-
-# encrypt_md5(string, [salt])
-# Returns a string encrypted in MD5 format
-sub encrypt_md5
-{
-local $passwd = $_[0];
-local $magic = '$1$';
-local $salt = $_[1] || substr(time(), -8);
-
-# Add the password, magic and salt
-local $cls = "MD5";
-eval "use MD5";
-if ($@) {
-	$cls = "Digest::MD5";
-	eval "use Digest::MD5";
-	if ($@) {
-		&error("Missing MD5 or Digest::MD5 perl modules");
-		}
-	}
-local $ctx = eval "new $cls";
-$ctx->add($passwd);
-$ctx->add($magic);
-$ctx->add($salt);
-
-# Add some more stuff from the hash of the password and salt
-local $ctx1 = eval "new $cls";
-$ctx1->add($passwd);
-$ctx1->add($salt);
-$ctx1->add($passwd);
-local $final = $ctx1->digest();
-for($pl=length($passwd); $pl>0; $pl-=16) {
-	$ctx->add($pl > 16 ? $final : substr($final, 0, $pl));
-	}
-
-# This piece of code seems rather pointless, but it's in the C code that
-# does MD5 in PAM so it has to go in!
-local $j = 0;
-local ($i, $l);
-for($i=length($passwd); $i; $i >>= 1) {
-	if ($i & 1) {
-		$ctx->add("\0");
-		}
-	else {
-		$ctx->add(substr($passwd, $j, 1));
-		}
-	}
-$final = $ctx->digest();
-
-# This loop exists only to waste time
-for($i=0; $i<1000; $i++) {
-	$ctx1 = eval "new $cls";
-	$ctx1->add($i & 1 ? $passwd : $final);
-	$ctx1->add($salt) if ($i % 3);
-	$ctx1->add($passwd) if ($i % 7);
-	$ctx1->add($i & 1 ? $final : $passwd);
-	$final = $ctx1->digest();
-	}
-
-# Convert the 16-byte final string into a readable form
-local $rv = $magic.$salt.'$';
-local @final = map { ord($_) } split(//, $final);
-$l = ($final[ 0]<<16) + ($final[ 6]<<8) + $final[12];
-$rv .= &to64($l, 4);
-$l = ($final[ 1]<<16) + ($final[ 7]<<8) + $final[13];
-$rv .= &to64($l, 4);
-$l = ($final[ 2]<<16) + ($final[ 8]<<8) + $final[14];
-$rv .= &to64($l, 4);
-$l = ($final[ 3]<<16) + ($final[ 9]<<8) + $final[15];
-$rv .= &to64($l, 4);
-$l = ($final[ 4]<<16) + ($final[10]<<8) + $final[ 5];
-$rv .= &to64($l, 4);
-$l = $final[11];
-$rv .= &to64($l, 2);
-
-return $rv;
-}
-
-@itoa64 = split(//, "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
-sub to64
-{
-local ($v, $n) = @_;
-local $r;
-while(--$n >= 0) {
-        $r .= $itoa64[$v & 0x3f];
-        $v >>= 6;
-        }
-return $r;
-}
-
 # encrypt_password(password)
 # Encrypts a password using the encryption format configured for this system
 sub encrypt_password
@@ -1414,7 +1328,20 @@ elsif ($access{'uedit_mode'} == 5) {
 elsif ($access{'uedit_mode'} == 6) {
 	@ulist = grep { $_->{'user'} eq $remote_user } @ulist;
 	}
-return @ulist;
+if ($access{'view'}) {
+	# Include non-editable users in results
+	local @rv = @{$_[1]};
+	local $u;
+	foreach $u (@rv) {
+		if (&indexof($u, @ulist) < 0) {
+			$u->{'noedit'} = 1;
+			}
+		}
+	return @rv;
+	}
+else {
+	return @ulist;
+	}
 }
 
 # list_allowed_groups(&access, &allgroups)
@@ -1442,7 +1369,20 @@ elsif ($access{'gedit_mode'} == 4) {
 		(!$access{'gedit2'} || $_->{'gid'} <= $access{'gedit2'})
 			} @glist;
 	}
-return @glist;
+if ($access{'view'}) {
+	# Include non-editable groups in results
+	local @rv = @{$_[1]};
+	local $g;
+	foreach $g (@rv) {
+		if (&indexof($g, @glist) < 0) {
+			$g->{'noedit'} = 1;
+			}
+		}
+	return @rv;
+	}
+else {
+	return @glist;
+	}
 }
 
 # batch_start()
@@ -1661,41 +1601,68 @@ while(@ginfo = &my_getgrent()) {
 	$gidgrp{$ginfo[2]} = $ginfo[0];
 	}
 &my_endgrent();
+local ($anyedit) = grep { !$_->{'noedit'} } @{$_[0]};
+local $lshow = $config{'last_show'};
 print "<form action=mass_delete_user.cgi method=post>\n";
+if ($anyedit) {
+	print &select_all_link("d", 0),"\n";
+	print &select_invert_link("d", 0),"<br>\n";
+	}
 print "<table width=100% border>\n";
 print "<tr $tb> ",
-      "<td><br></td> ",
+      !$anyedit ? "" : "<td><br></td> ",
       "<td><b>$text{'user'}</b></td> ",
       "<td><b>$text{'uid'}</b></td> ",
       "<td><b>$text{'gid'}</b></td> ",
       "<td><b>$text{'real'}</b></td> ",
       "<td><b>$text{'home'}</b></td> ",
-      "<td><b>$text{'shell'}</b></td> </tr>\n";
+      "<td><b>$text{'shell'}</b></td> ",
+      ($lshow ? "<td><b>$text{'lastlogin'}</b></td> " : ""),
+      "</tr>\n";
+local %llogin;
+if ($lshow) {
+	local $l;
+	foreach $l (&list_last_logins()) {
+		$llogin{$l->[0]} ||= $l->[3];
+		}
+	}
 local $u;
 foreach $u (@{$_[0]}) {
 	$u->{'real'} =~ s/,.*$// if ($config{'extra_real'});
 	print "<tr $cb> ",
-	      "<td><input type=checkbox name=d ",
-	      "value='$u->{'user'}'></td> ",
-	      "<td><a href='edit_user.cgi?",
-	      "num=$u->{'num'}'>",&html_escape($u->{'user'}),
-	      "</a></td> <td>$u->{'uid'}</td> ",
+	      !$anyedit ? "" :
+	      $u->{'noedit'} ? "<td></td>" :
+	        "<td><input type=checkbox name=d value='$u->{'user'}'></td> ",
+	      "<td>",&user_link($u),"</td>",
+	      "<td>$u->{'uid'}</td> ",
 	      "<td>",$gidgrp{$u->{'gid'}}||$u->{'gid'},"</td> ",
 	      "<td>",&ifblank($u->{'real'}),"</td> ",
 	      "<td>",&ifblank($u->{'home'}),"</td> ",
-	      "<td>",&ifblank($u->{'shell'}),"</td> </tr>\n";
+	      "<td>",&ifblank($u->{'shell'}),"</td> ",
+	      ($lshow ? "<td>".&ifblank($llogin{$u->{'user'}})."</td> " : ""),
+	      "</tr>\n";
 	}
 print "</table>\n";
-print "<input type=submit value='$text{'index_mass'}'>\n";
+if ($anyedit) {
+	print &select_all_link("d", 0),"\n";
+	print &select_invert_link("d", 0),"<br>\n";
+	}
+print "<input type=submit value='$text{'index_mass'}'>\n" if ($anyedit);
 print "</form>\n";
 }
 
 # groups_table(&groups)
 sub groups_table
 {
+local ($anyedit) = grep { !$_->{'noedit'} } @{$_[0]};
 print "<form action=mass_delete_group.cgi method=post>\n";
 print "<table border width=100%>\n";
-print "<tr $tb> <td><br></td> ",
+if ($anyedit) {
+	print &select_all_link("d", 0),"\n";
+	print &select_invert_link("d", 0),"<br>\n";
+	}
+print "<tr $tb> ",
+      !$anyedit ? "" : "<td><br></td> ",
       "<td><b>$text{'gedit_group'}</b></td> ",
       "<td><b>$text{'gedit_gid'}</b></td> ",
       "<td><b>$text{'gedit_members'}</b></td> </tr>\n";
@@ -1703,14 +1670,19 @@ local $g;
 foreach $g (@{$_[0]}) {
 	local $members = join(" ", split(/,/, $g->{'members'}));
 	print "<tr $cb>\n";
-	print "<td><input type=checkbox name=d value='$g->{'group'}'></td>\n";
-	print "<td><a href='edit_group.cgi?",
-	      "num=$g->{'num'}'>",&html_escape($g->{'group'}),
-	      "</a></td> <td>$g->{'gid'}</td> ",
+	print !$anyedit ? "" :
+	      $g->{'noedit'} ? "<td></td>" :
+	       "<td><input type=checkbox name=d value='$g->{'group'}'></td>\n";
+	print "<td>",&group_link($g),"</td> ",
+	      "<td>$g->{'gid'}</td> ",
 	      "<td>",&ifblank($members),"</td> </tr>\n";
 	}
 print "</table>\n";
-print "<input type=submit value='$text{'index_gmass'}'>\n";
+if ($anyedit) {
+	print &select_all_link("d", 0),"\n";
+	print &select_invert_link("d", 0),"<br>\n";
+	}
+print "<input type=submit value='$text{'index_gmass'}'>\n" if ($anyedit);
 print "</form>\n";
 }
 
@@ -1734,4 +1706,46 @@ print "/<input name=$_[3]y size=5 value='$_[2]'>";
 print &date_chooser_button("$_[3]d", "$_[3]m", "$_[3]y");
 }
 
+# list_last_logins([user], [max])
+# Returns a list of array references, each containing the details of a login
+sub list_last_logins
+{
+local @rv;
+&open_last_command(LAST, $_[0]);
+while(@last = &read_last_line(LAST)) {
+	push(@rv, [ @last ]);
+	if ($_[1] && scalar(@rv) >= $_[1]) {
+		last;	# reached max
+		}
+	}
+close(LAST);
+return @rv;
+}
+
+# user_link(&user)
+sub user_link
+{
+if ($_[0]->{'noedit'}) {
+	return &html_escape($_[0]->{'user'});
+	}
+else {
+	return "<a href='edit_user.cgi?num=$_[0]->{'num'}'>".
+	       &html_escape($_[0]->{'user'})."</a>";
+	}
+}
+
+# group_link(&group)
+sub group_link
+{
+if ($_[0]->{'noedit'}) {
+	return &html_escape($_[0]->{'group'});
+	}
+else {
+	return "<a href='edit_group.cgi?num=$_[0]->{'num'}'>".
+	       &html_escape($_[0]->{'group'})."</a>";
+	}
+}
+
+
+
 1;
