@@ -28,29 +28,35 @@ $third_ssl = 0;
 
 $default_key_size = "512";
 
+$cron_cmd = "$module_config_directory/update.pl";
+
 sub setup_ca
 {
 local $adir = &module_root_directory("acl");
 local $conf = `cat $adir/openssl.cnf`;
 local $acl = "$config_directory/acl";
 $conf =~ s/DIRECTORY/$acl/g;
+
 &lock_file("$acl/openssl.cnf");
-open(CONF, ">$acl/openssl.cnf");
-print CONF $conf;
-close(CONF);
+&open_tempfile(CONF, ">$acl/openssl.cnf");
+&print_tempfile(CONF, $conf);
+&close_tempfile(CONF);
 chmod(0600, "$acl/openssl.cnf");
 &unlock_file("$acl/openssl.cnf");
+
 &lock_file("$acl/index.txt");
-open(INDEX, ">$acl/index.txt");
-close(INDEX);
+&open_tempfile(INDEX, ">$acl/index.txt");
+&close_tempfile(INDEX);
 chmod(0600, "$acl/index.txt");
 &unlock_file("$acl/index.txt");
+
 &lock_file("$acl/serial");
-open(SERIAL, ">$acl/serial");
-print SERIAL "011E\n";
-close(SERIAL);
+&open_tempfile(SERIAL, ">$acl/serial");
+&print_tempfile(SERIAL, "011E\n");
+&close_tempfile(SERIAL);
 chmod(0600, "$acl/serial");
 &unlock_file("$acl/serial");
+
 &lock_file("$acl/newcerts");
 mkdir("$acl/newcerts", 0700);
 chmod(0700, "$acl/newcerts");
@@ -101,8 +107,8 @@ if ($two eq "\037\235") {
 		unlink($file) if ($need_unlink);
 		return &text('install_ecomp', "<tt>uncompress</tt>");
 		}
-	local $temp = $file =~ /\/([^\/]+)\.Z/i ? &tempname("$1")
-						: &tempname();
+	local $temp = $file =~ /\/([^\/]+)\.Z/i ? &transname("$1")
+						: &transname();
 	local $out = `uncompress -c "$file" 2>&1 >$temp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
@@ -117,8 +123,8 @@ elsif ($two eq "\037\213") {
 		unlink($file) if ($need_unlink);
 		return &text('install_egzip', "<tt>gunzip</tt>");
 		}
-	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &tempname("$1")
-						 : &tempname();
+	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &transname("$1")
+						 : &transname();
 	local $out = `gunzip -c "$file" 2>&1 >$temp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
@@ -133,8 +139,8 @@ elsif ($two eq "BZ") {
 		unlink($file) if ($need_unlink);
 		return &text('install_ebunzip', "<tt>bunzip2</tt>");
 		}
-	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &tempname("$1")
-						 : &tempname();
+	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &transname("$1")
+						 : &transname();
 	local $out = `bunzip2 -c "$file" 2>&1 >$temp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
@@ -219,7 +225,7 @@ else {
 
 	# Get the module.info files to check dependancies
 	local $ver = &get_webmin_version();
-	local $tmpdir = &tempname();
+	local $tmpdir = &transname();
 	mkdir($tmpdir, 0700);
 	local $err;
 	local @realmods;
@@ -354,16 +360,16 @@ sub grant_user_module
 # Grant to appropriate users
 local %acl;
 &read_acl(undef, \%acl);
-open(ACL, ">".&acl_filename()); 
+&open_tempfile(ACL, ">".&acl_filename()); 
 local $u;
 foreach $u (keys %acl) {
 	local @mods = @{$acl{$u}};
 	if (!$_[0] || &indexof($u, @{$_[0]}) >= 0) {
 		@mods = &unique(@mods, @{$_[1]});
 		}
-	print ACL "$u: ",join(' ', @mods),"\n";
+	&print_tempfile(ACL, "$u: ",join(' ', @mods),"\n");
 	}
-close(ACL);
+&close_tempfile(ACL);
 
 # Grant to appropriate groups
 if ($_[1] && &foreign_check("acl")) {
@@ -528,7 +534,7 @@ return 0;
 # message instead.
 sub list_standard_modules
 {
-local $temp = &tempname();
+local $temp = &transname();
 local $error;
 local ($host, $port, $page, $ssl);
 if ($config{'standard_url'}) {
@@ -566,7 +572,7 @@ return "<input type=button onClick='ifield = document.forms[$form].$_[0]; choose
 # occurs, returns the message instead.
 sub list_third_modules
 {
-local $temp = &tempname();
+local $temp = &transname();
 local $error;
 local ($host, $port, $page, $ssl);
 if ($config{'third_url'}) {
@@ -639,11 +645,11 @@ sub save_newmodule_users
 {
 &lock_file($newmodule_users_file);
 if ($_[0]) {
-	open(NEWMODS, ">$newmodule_users_file");
+	&open_tempfile(NEWMODS, ">$newmodule_users_file");
 	foreach $u (@{$_[0]}) {
-		print NEWMODS "$u\n";
+		&print_tempfile(NEWMODS, "$u\n");
 		}
-	close(NEWMODS);
+	&close_tempfile(NEWMODS);
 	}
 else {
 	unlink($newmodule_users_file);
@@ -681,7 +687,7 @@ sub fetch_updates
 local ($host, $port, $page, $ssl) = &parse_http_url($_[0]);
 $host || &error($text{'update_eurl'});
 
-local $temp = &tempname();
+local $temp = &transname();
 local @updates;
 &http_download($host, $port, $page, $temp, undef, undef, $ssl);
 open(UPDATES, $temp);
@@ -697,4 +703,104 @@ unlink($temp);
 return ( \@updates, $host, $port, $page, $ssl );
 }
 
+# find_cron_job(\@jobs)
+sub find_cron_job
+{
+local ($job) = grep { $_->{'user'} eq 'root' &&
+		      $_->{'command'} eq $cron_cmd } @{$_[0]};
+return $job;
+}
+
+# get_ipkeys(&miniserv)
+# Returns a list of IP address to key file mappings from a miniserv.conf entry
+sub get_ipkeys
+{
+local (@rv, $k);
+foreach $k (keys %{$_[0]}) {
+	if ($k =~ /^ipkey_(\S+)/) {
+		local $ipkey = { 'ips' => [ split(/,/, $1) ],
+				 'key' => $_[0]->{$k},
+				 'index' => scalar(@rv) };
+		$ipkey->{'cert'} = $_[0]->{'ipcert_'.$1};
+		push(@rv, $ipkey);
+		}
+	}
+return @rv;
+}
+
+# save_ipkeys(&miniserv, &keys)
+# Updates miniserv.conf entries from the given list of keys
+sub save_ipkeys
+{
+local $k;
+foreach $k (keys %{$_[0]}) {
+	if ($k =~ /^(ipkey_|ipcert_)/) {
+		delete($_[0]->{$k});
+		}
+	}
+foreach $k (@{$_[1]}) {
+	local $ips = join(",", @{$k->{'ips'}});
+	$_[0]->{'ipkey_'.$ips} = $k->{'key'};
+	if ($k->{'cert'}) {
+		$_[0]->{'ipcert_'.$ips} = $k->{'cert'};
+		}
+	}
+}
+
+# validate_key_cert(key, [cert])
+# Call &error if some key and cert file don't look correct
+sub validate_key_cert
+{
+local $key = `cat '$_[0]' 2>/dev/null`;
+$key =~ /BEGIN RSA PRIVATE KEY/i || &error(&text('ssl_ekey', $_[0]));
+if (!$_[1]) {
+	$key =~ /BEGIN CERTIFICATE/ || &error(&text('ssl_ecert', $_[0]));
+	}
+else {
+	local $cert = `cat '$_[1]' 2>/dev/null`;
+	$cert =~ /BEGIN CERTIFICATE/ || &error(&text('ssl_ecert', $_[1]));
+	}
+}
+
+# detect_operating_system([os-list-file])
+# Returns a hash containing os_type, os_version, real_os_type and
+# real_os_version
+sub detect_operating_system
+{
+local $file = $_[0] || "$root_directory/os_list.txt";
+local $temp = &transname();
+local $perl = &get_perl_path();
+system("$perl $root_directory/oschooser.pl $file $temp 1");
+local %rv;
+&read_env_file($temp, \%rv);
+return %rv;
+}
+
+# shared_root_directory()
+# Returns 1 if the Webmin root directory is shared with another system, such as
+# via NFS, or in a Solaris zone. If so, updates and module installs are not
+# allowed.
+sub shared_root_directory
+{
+return 1 if ($gconfig{'shared_root'});
+if (&running_in_zone()) {
+	# In a Solaris zone .. is the root directory loopback mounted?
+	if (&foreign_exists("mount")) {
+		&foreign_require("mount", "mount-lib.pl");
+		local @rst = stat($root_directory);
+		local $m;
+		foreach $m (&mount::list_mounted()) {
+			local @mst = stat($m->[0]);
+			if ($mst[0] == $rst[0]) {
+				# Found the mount!
+				if ($m->[2] eq "lofs" || $m->[2] eq "nfs") {
+					return 1;
+					}
+				}
+			}
+		}
+	}
+return 0;
+}
+
 1;
